#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SeedEditor for organ segmentation

Example:

$ seed_editor_qp.py -f head.mat
"""

# import unittest
from optparse import OptionParser
from scipy.io import loadmat
import numpy as np
import sys
from scipy.spatial import Delaunay

from PyQt4.QtCore import Qt, QSize, QString, SIGNAL
from PyQt4.QtGui import QImage, QDialog,\
    QApplication, QSlider, QPushButton,\
    QLabel, QPixmap, QPainter, qRgba,\
    QComboBox, QIcon, QStatusBar,\
    QHBoxLayout, QVBoxLayout, QFrame, QSizePolicy

# BGRA order
GRAY_COLORTABLE = np.array([[ii, ii, ii, 255] for ii in range(256)],
                           dtype=np.uint8)

SEEDS_COLORTABLE = np.array([[0, 255, 0, 255],
                             [0, 0, 255, 255]], dtype=np.uint8)

CONTOURS_COLORS = {
    1: [255, 0, 0],
    2: [0, 0, 255],
    3: [0, 255, 255],
    4: [255, 128, 0],
    5: [255, 128, 0],
    6: [255, 0, 128],
    7: [128, 128, 255],
    8: [0, 128, 255],
    9: [128, 0, 255],
    10: [255, 255, 0],
    11: [0, 255, 0],
    12: [0, 255, 255],
    13: [0, 0, 255],
}

CONTOURS_COLORTABLE = np.zeros((256,4), dtype=np.uint8)
CONTOURS_COLORTABLE[:,:3] = 255
CONTOURLINES_COLORTABLE = np.zeros((256,2,4), dtype=np.uint8)
CONTOURLINES_COLORTABLE[:,:,:3] = 255

for ii, jj in CONTOURS_COLORS.iteritems():
    key = ii - 1
    CONTOURS_COLORTABLE[key,:3] = jj
    CONTOURS_COLORTABLE[key,3] = 64
    CONTOURLINES_COLORTABLE[key,0,:3] = jj
    CONTOURLINES_COLORTABLE[key,0,3] = 16
    CONTOURLINES_COLORTABLE[key,1,:3] = jj
    CONTOURLINES_COLORTABLE[key,1,3] = 255

VIEW_TABLE = {'axial': (2,1,0),
              'sagittal': (1,0,2),
              'coronal': (2,0,1)}
DRAW_MASK = [
    (np.array([[1]], dtype=np.int8), 'small pen'),
    (np.array([[0, 1, 1, 1, 0],
               [1, 1, 1, 1, 1],
               [1, 1, 1, 1, 1],
               [1, 1, 1, 1, 1],
               [0, 1, 1, 1, 0]], dtype=np.int8), 'middle pen'),
    (np.array([[0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0],
               [0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0],
               [0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0],
               [0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0],
               [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
               [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
               [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
               [0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0],
               [0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0],
               [0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0],
               [0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0]], dtype=np.int8), 'large pen'),
    ]

BOX_BUTTONS_SEED = {
    Qt.LeftButton: 1,
    Qt.RightButton: 2,
    }

BOX_BUTTONS_DRAW = {
    Qt.LeftButton: 1,
    Qt.RightButton: 0,
    }

NEI_TAB = [[-1, -1], [0, -1], [1, -1],
           [-1, 0], [1, 0],
           [-1, 1], [0, 1], [1, 1]]


def erase_reg(arr, p, val=0):
    from scipy.ndimage.measurements import label

    labs, num = label(arr)
    aval = labs[p]
    idxs = np.where(labs == aval)
    arr[idxs] = val

class SliceBox(QLabel):
    """
    Widget for marking reagions of interest in DICOM slices.
    """

    def __init__(self, sliceSize, grid,
                 mode='seeds'):
        """
        Initialize SliceBox.

        Parameters
        ----------
        sliceSize : tuple of int
            Size of slice matrix.
        grid : tuple of float
            Pixel size:
            imageSize = (grid1 * sliceSize1, grid2 * sliceSize2)
        mode : str
            Editor mode.
        """

        QLabel.__init__(self)

        self.drawing = False
        self.modified = False
        self.seed_mark = None
        self.last_position = None
        self.imagesize = QSize(int(sliceSize[0] * grid[0]),
                               int(sliceSize[1] * grid[1]))
        self.grid = grid
        self.slice_size = sliceSize
        self.ctslice_rgba = None
        self.cw = {'c': 1.0, 'w': 1.0}

        self.seeds = None
        self.contours = None
        self.contours_old = None
        self.mask_points = None
        self.erase_region_button = None
        self.erase_fun = None
        self.erase_mode = 'inside'
        self.contour_mode = 'fill'
        self.scroll_fun = None

        if mode == 'draw':
            self.seeds_colortable = CONTOURS_COLORTABLE
            self.box_buttons = BOX_BUTTONS_DRAW
            self.mode_draw = True

        else:
            self.seeds_colortable = SEEDS_COLORTABLE
            self.box_buttons = BOX_BUTTONS_SEED
            self.mode_draw = False

        self.image = QImage(self.imagesize, QImage.Format_RGB32)
        self.setPixmap(QPixmap.fromImage(self.image))
        self.setScaledContents(True)

    def paintEvent(self, event):
        painter = QPainter(self)
        painter.drawImage(event.rect(), self.image)
        painter.end()

    def drawSeedMark(self, x, y):
        xx = self.mask_points[0] + x
        yy = self.mask_points[1] + y
        idx = np.arange(len(xx))
        idx[np.where(xx < 0)] = -1
        idx[np.where(xx >= self.slice_size[0])] = -1
        idx[np.where(yy < 0)] = -1
        idx[np.where(yy >= self.slice_size[1])] = -1
        ii = idx[np.where(idx >= 0)]
        xx = xx[ii]
        yy = yy[ii]

        self.seeds[yy * self.slice_size[0] + xx] = self.seed_mark

    def drawLine(self, p0, p1):
        """
        Draw line to slice image and seed matrix.

        Parameters
        ----------
        p0 : tuple of int
            Line star point.
        p1 : tuple of int
            Line end point.
        """

        x0, y0 = p0
        x1, y1 = p1
        dx = np.abs(x1-x0)
        dy = np.abs(y1-y0)
        if x0 < x1:
            sx = 1

        else:
            sx = -1

        if y0 < y1:
            sy = 1

        else:
            sy = -1

        err = dx - dy

        while True:
            self.drawSeedMark(x0,y0)

            if x0 == x1 and y0 == y1:
                break

            e2 = 2*err
            if e2 > -dy:
                err = err - dy
                x0 = x0 + sx

            if e2 <  dx:
                err = err + dx
                y0 = y0 + sy

    def drawSeeds(self, pos):
        if pos[0] < 0 or pos[0] >= self.slice_size[0] \
                or pos[1] < 0 or pos[1] >= self.slice_size[1]:
            return

        self.drawLine(self.last_position, pos)
        self.updateSlice()

        self.modified = True
        self.last_position = pos

        self.update()

    def get_contours(self, img, sl):
        idxs = sl.nonzero()[0]
        keys = np.unique(sl[idxs])
        for ii in keys:
            if ii == 0:
                continue
            aux = np.zeros_like(sl)
            idxsi = np.where(sl == ii)[0]
            aux[idxsi] = 1
            cnt = self.gen_contours(aux)

            self.composeRgba(img, cnt,
                             CONTOURLINES_COLORTABLE[ii - 1,...])


    def gen_contours(self, sl):
        sls = sl.reshape(self.slice_size, order='F')
        cnt = sls.copy()
        chunk = np.zeros((cnt.shape[1] + 2,), dtype=np.int8)
        for irow, row in enumerate(sls):
            chunk[1:-1] = row
            chdiff = np.diff(chunk)
            idx1 = np.where(chdiff > 0)[0]
            if idx1.shape[0] > 0:
                idx2 = np.where(chdiff < 0)[0]
                if idx2.shape[0] > 0:
                    cnt[irow,idx1] = 2
                    cnt[irow,idx2 - 1] = 2

        chunk = np.zeros((cnt.shape[0] + 2,), dtype=np.int8)
        for icol, col in enumerate(sls.T):
            chunk[1:-1] = col
            chdiff = np.diff(chunk)
            idx1 = np.where(chdiff > 0)[0]
            if idx1.shape[0] > 0:
                idx2 = np.where(chdiff < 0)[0]
                if idx2.shape[0] > 0:
                    cnt[idx1,icol] = 2
                    cnt[idx2 - 1,icol] = 2

        return cnt.ravel(order='F')

    def composeRgba(self, bg, fg, cmap):
        idxs = fg.nonzero()[0]

        if idxs.shape[0] > 0:
            fg_rgb = cmap[fg[idxs] - 1]

            af = fg_rgb[...,3].astype(np.uint32)
            rgbf = fg_rgb[...,:3].astype(np.uint32)
            rgbb = bg[idxs,:3].astype(np.uint32)

            rgbx = ((rgbf.T * af).T + (rgbb.T * (255 - af)).T) / 255
            bg[idxs,:3] = rgbx.astype(np.uint8)

    def overRgba(self, bg, fg, cmap):
        idxs = fg.nonzero()[0]
        bg[idxs] = cmap[fg[idxs] - 1]

    def updateSlice(self):

        if self.ctslice_rgba is None:
            return

        img = self.ctslice_rgba.copy()

        if self.seeds is not None:
            if self.mode_draw:
                if self.contour_mode == 'fill':
                    self.composeRgba(img, self.seeds,
                                     self.seeds_colortable)
                elif self.contour_mode == 'contours':
                    self.get_contours(img, self.seeds)

            else:
                self.overRgba(img, self.seeds,
                              self.seeds_colortable)

        if self.contours is not None:
            if self.contour_mode == 'fill':
                self.composeRgba(img, self.contours,
                                 CONTOURS_COLORTABLE)

            elif self.contour_mode == 'contours':
                self.get_contours(img, self.contours)

        image = QImage(img.flatten(),
                     self.slice_size[0], self.slice_size[1],
                     QImage.Format_ARGB32).scaled(self.imagesize)
        painter = QPainter(self.image)
        painter.drawImage(0, 0, image)
        painter.end()

        self.update()

    def getSliceRGBA(self, ctslice):
        if self.cw['w'] > 0:
            mul = 255.0 / float(self.cw['w'])

        else:
            mul = 0

        lb = self.cw['c'] - self.cw['w'] / 2
        aux = (ctslice.ravel(order='F') - lb) * mul
        idxs = np.where(aux < 0)[0]
        aux[idxs] = 0
        idxs = np.where(aux > 255)[0]
        aux[idxs] = 255

        return aux.astype(np.uint8)

    def updateSliceCW(self, ctslice=None):
        if ctslice is not None:
            self.ctslice_rgba = GRAY_COLORTABLE[self.getSliceRGBA(ctslice)]

        self.updateSlice()

    def setSlice(self, ctslice=None, seeds=None, contours=None):
        if ctslice is not None:
            self.ctslice_rgba = GRAY_COLORTABLE[self.getSliceRGBA(ctslice)]

        if seeds is not None:
            self.seeds = seeds.ravel(order='F')

        else:
            self.seeds = None

        if contours is not None:
            self.contours = contours.ravel(order='F')

        else:
            self.contours = None

        self.updateSlice()

    def getSliceSeeds(self):
        if self.modified:
            self.modified = False
            return self.seeds.reshape(self.slice_size, order='F')

        else:
            return None

    def gridPosition(self, pos):
        return (int(pos.x() / self.grid[0]),
                int(pos.y() / self.grid[1]))

    # mouse events
    def mousePressEvent(self, event):
        if event.button() in self.box_buttons:
            self.drawing = True
            self.seed_mark = self.box_buttons[event.button()]
            self.last_position = self.gridPosition(event.pos())

        elif event.button() == Qt.MiddleButton:
            self.drawing = False
            self.erase_region_button = True

    def mouseMoveEvent(self, event):
        if self.drawing:
            self.drawSeeds(self.gridPosition(event.pos()))

    def mouseReleaseEvent(self, event):
        if (event.button() in self.box_buttons) and self.drawing:
            self.drawSeeds(self.gridPosition(event.pos()))
            self.drawing = False

        if event.button() == Qt.MiddleButton\
          and self.erase_region_button == True:
            self.eraseRegion(self.gridPosition(event.pos()),
                             self.erase_mode)

            self.erase_region_button == False

    def resizeSlice(self, new_slice_size=None, new_grid=None):

        if new_slice_size is not None:
            self.slice_size = new_slice_size

        if new_grid is not None:
            self.grid = new_grid

        self.imagesize = QSize(int(self.slice_size[0] * self.grid[0]),
                               int(self.slice_size[1] * self.grid[1]))
        self.image = QImage(self.imagesize, QImage.Format_RGB32)
        self.setPixmap(QPixmap.fromImage(self.image))

    def resizeEvent(self, event):
        new_height = self.height()
        new_grid = new_height / float(self.slice_size[1])
        mul = new_grid / self.grid[1]

        self.grid = np.array(self.grid) * mul
        self.resizeSlice()
        self.updateSlice()

    def leaveEvent(self, event):
        self.drawing = False

    def enterEvent(self, event):
        self.drawing = False
        self.emit(SIGNAL('focus_slider'))

    def setMaskPoints(self, mask):
        self.mask_points = mask

    def getCW(self):
        return self.cw

    def setCW(self, val, key):
        self.cw[key] = val

    def eraseRegion(self, pos, mode):
        if self.erase_fun is not None:
            self.erase_fun(pos, mode)
            self.updateSlice()

    def setEraseFun(self, fun):
        self.erase_fun = fun

    def setScrollFun(self, fun):
        self.scroll_fun = fun

    def wheelEvent(self, event):
        d = event.delta()
        nd = d / abs(d)
        if self.scroll_fun is not None:
            self.scroll_fun(-nd)

class QTSeedEditor(QDialog):
    """
    DICOM viewer.
    """
    @staticmethod
    def get_line(mode='h'):
        line = QFrame()
        if mode == 'h':
            line.setFrameStyle(QFrame.HLine)
        elif mode == 'v':
            line.setFrameStyle(QFrame.VLine)

        line.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Expanding)

        return line

    def initUI(self, shape, vscale, height=600,
               mode='seed'):
        """
        Initialize UI.

        Parameters
        ----------
        shape : (int, int, int)
            Shape of data matrix.
        vscale : (float, float, float)
            Voxel scaling.
        height : int
            Maximal slice height in pixels.
        mode : str
            Editor mode.
        """

        # picture
        grid = height / float(shape[1] * vscale[1])
        mgrid = (grid * vscale[0], grid * vscale[1])
        self.slice_box = SliceBox(shape[:-1], mgrid,
                                  mode)
        self.slice_box.setScrollFun(self.scrollSlices)
        self.connect(self.slice_box, SIGNAL('focus_slider'), self.focusSliceSlider)

        # sliders
        self.allow_select_slice = True
        self.n_slices = shape[2]
        self.slider = QSlider(Qt.Vertical)
        self.slider.label = QLabel()
        self.slider.label.setText('Slice: %d / %d' % (self.actual_slice,
                                                      self.n_slices))
        self.slider.setRange(1, self.n_slices)
        self.slider.valueChanged.connect(self.sliderSelectSlice)
        self.slider.setValue(self.actual_slice)

        self.slider_cw = {}
        self.slider_cw['c'] = QSlider(Qt.Horizontal)
        self.slider_cw['c'].valueChanged.connect(self.changeC)
        self.slider_cw['c'].label = QLabel()

        self.slider_cw['w'] = QSlider(Qt.Horizontal)
        self.slider_cw['w'].valueChanged.connect(self.changeW)
        self.slider_cw['w'].label = QLabel()

        self.view_label = QLabel('View size: %d x %d' % self.img_aview.shape[:-1])
        self.voxel_label = QLabel('Voxel size [mm]:\n  %.2f x %.2f x %.2f'\
                                      % tuple(self.voxel_size[np.array(self.act_transposition)]))

        combo_view_options = VIEW_TABLE.keys()
        combo_view = QComboBox(self)
        combo_view.activated[str].connect(self.setView)
        combo_view.addItems(combo_view_options)

        # buttons
        self.btn_quit = QPushButton("Return", self)
        self.btn_quit.clicked.connect(self.quit)

        combo_dmask = QComboBox(self)
        combo_dmask.activated.connect(self.changeMask)
        self.mask_points_tab, aux = self.init_draw_mask(DRAW_MASK, mgrid)
        for icon, label in aux:
            combo_dmask.addItem(icon, label)


        self.slice_box.setMaskPoints(self.mask_points_tab[combo_dmask.currentIndex()])

        self.status_bar = QStatusBar()

        vopts = []
        vmenu = []
        appmenu = []
        if mode == 'seed' and self.mode_fun is not None:
            btn_recalc = QPushButton("Recalculate", self)
            btn_recalc.clicked.connect(self.recalculate)
            appmenu.append(QLabel('<b>Segmentation mode</b><br><br><br>' +
                                  'Select the region of interest<br>' +
                                  'using the mouse buttons:<br><br>' +
                                  '&nbsp;&nbsp;<i>left</i> - inner region<br>' +
                                  '&nbsp;&nbsp;<i>right</i> - outer region<br><br>'))
            appmenu.append(btn_recalc)
            appmenu.append(QLabel())
            self.volume_label = QLabel('Volume:\n  unknown')
            appmenu.append(self.volume_label)

            # Set middle pencil as default (M. Jirik)
            combo_dmask.setCurrentIndex(1)
            self.slice_box.setMaskPoints(
                self.mask_points_tab[combo_dmask.currentIndex()])
            #  -----mjirik---end------

        if mode == 'seed' or mode == 'crop'\
                or mode == 'mask' or mode == 'draw':
            btn_del = QPushButton("Delete Seeds", self)
            btn_del.clicked.connect(self.deleteSliceSeeds)
            vmenu.append(None)
            vmenu.append(btn_del)

            combo_contour_options = ['fill', 'contours', 'off']
            combo_contour = QComboBox(self)
            combo_contour.activated[str].connect(self.changeContourMode)
            combo_contour.addItems(combo_contour_options)
            self.changeContourMode(combo_contour_options[combo_contour.currentIndex()])
            vopts.append(QLabel('Selection mode:'))
            vopts.append(combo_contour)


        if mode == 'mask':
            btn_recalc_mask = QPushButton("Recalculate mask", self)
            btn_recalc_mask.clicked.connect(self.updateMaskRegion_btn)
            btn_all = QPushButton("Select all", self)
            btn_all.clicked.connect(self.maskSelectAll)
            btn_reset = QPushButton("Reset selection", self)
            btn_reset.clicked.connect(self.resetSelection)
            btn_reset_seads = QPushButton("Reset seads", self)
            btn_reset_seads.clicked.connect(self.resetSeads)
            btn_add = QPushButton("Add selection", self)
            btn_add.clicked.connect(self.maskAddSelection)
            btn_rem = QPushButton("Remove selection", self)
            btn_rem.clicked.connect(self.maskRemoveSelection)
            btn_mask = QPushButton("Mask region", self)
            btn_mask.clicked.connect(self.maskRegion)
            appmenu.append(QLabel('<b>Mask mode</b><br><br><br>' +
                                  'Select the region to mask<br>' +
                                  'using the left mouse button<br><br>'))
            appmenu.append(self.get_line('h'))
            appmenu.append(btn_recalc_mask)
            appmenu.append(btn_all)
            appmenu.append(btn_reset)
            appmenu.append(btn_reset_seads)
            appmenu.append(self.get_line('h'))
            appmenu.append(btn_add)
            appmenu.append(btn_rem)
            appmenu.append(self.get_line('h'))
            appmenu.append(btn_mask)
            appmenu.append(self.get_line('h'))
            self.mask_qhull = None

        if mode == 'crop':
            btn_crop = QPushButton("Crop", self)
            btn_crop.clicked.connect(self.crop)
            appmenu.append(QLabel('<b>Crop mode</b><br><br><br>' +
                                  'Select the crop region<br>' +
                                  'using the left mouse button<br><br>'))
            appmenu.append(btn_crop)

        if mode == 'draw':
            appmenu.append(QLabel('<b>Manual segmentation<br> mode</b><br><br><br>' +
                                  'Mark the region of interest<br>' +
                                  'using the mouse buttons:<br><br>' +
                                  '&nbsp;&nbsp;<i>left</i> - draw<br>' +
                                  '&nbsp;&nbsp;<i>right</i> - erase<br>' +
                                  '&nbsp;&nbsp;<i>middle</i> - vol. erase<br><br>'))

            btn_reset = QPushButton("Reset", self)
            btn_reset.clicked.connect(self.resetSliceDraw)
            vmenu.append(None)
            vmenu.append(btn_reset)

            combo_erase_options = ['inside', 'outside']
            combo_erase = QComboBox(self)
            combo_erase.activated[str].connect(self.changeEraseMode)
            combo_erase.addItems(combo_erase_options)
            self.changeEraseMode(combo_erase_options[combo_erase.currentIndex()])
            vopts.append(QLabel('Volume erase mode:'))
            vopts.append(combo_erase)

        hbox = QHBoxLayout()
        vbox = QVBoxLayout()
        vbox_left = QVBoxLayout()
        vbox_app = QVBoxLayout()

        hbox.addWidget(self.slice_box)
        hbox.addWidget(self.slider)
        vbox_left.addWidget(self.slider.label)
        vbox_left.addWidget(self.view_label)
        vbox_left.addWidget(self.voxel_label)
        vbox_left.addWidget(QLabel())
        vbox_left.addWidget(QLabel('View plane:'))
        vbox_left.addWidget(combo_view)
        vbox_left.addWidget(self.get_line())
        vbox_left.addWidget(self.slider_cw['c'].label)
        vbox_left.addWidget(self.slider_cw['c'])
        vbox_left.addWidget(self.slider_cw['w'].label)
        vbox_left.addWidget(self.slider_cw['w'])
        vbox_left.addWidget(self.get_line())
        vbox_left.addWidget(QLabel('Drawing mask:'))
        vbox_left.addWidget(combo_dmask)

        for ii in vopts:
            vbox_left.addWidget(ii)

        for ii in vmenu:
            if ii is None:
                vbox_left.addStretch(1)

            else:
                vbox_left.addWidget(ii)

        for ii in appmenu:
            if ii is None:
                vbox_app.addStretch(1)

            else:
                vbox_app.addWidget(ii)

        vbox_app.addStretch(1)
        vbox_app.addWidget(self.btn_quit)

        hbox.addLayout(vbox_left)
        hbox.addWidget(self.get_line('v'))
        hbox.addLayout(vbox_app)
        vbox.addLayout(hbox)
        vbox.addWidget(self.status_bar)
        self.my_layout = vbox
        self.setLayout(vbox)

        self.setWindowTitle('Segmentation Editor')
        self.show()

    def __init__(self, img, viewPositions=None,
                 seeds=None, contours=None,
                 mode='seed', modeFun=None,
                 voxelSize=[1,1,1], volume_unit='mm3'):
        """
        Initiate Editor

        Parameters
        ----------
        img : array
            DICOM data matrix.
        actualSlice : int
            Index of actual slice.
        seeds : array
            Seeds, user defined regions of interest.
        contours : array
            Computed segmentation.
        mode : str
            Editor modes:
               'seed' - seed editor
               'crop' - manual crop
               'draw' - drawing
               'mask' - mask region
        modeFun : fun
            Mode function invoked by user button.
        voxelSize : tuple of float
            voxel size [mm]
        volume_unit : allow select output volume in mililiters or mm3
            [mm, ml]
        """

        QDialog.__init__(self)

        self.mode = mode
        self.mode_fun = modeFun

        self.actual_view = 'axial'
        self.act_transposition = VIEW_TABLE[self.actual_view]
        self.img = img
        self.img_aview = self.img.transpose(self.act_transposition)

        self.volume_unit = volume_unit

        self.last_view_position = {}
        for jj, ii in enumerate(VIEW_TABLE.iterkeys()):
            if viewPositions is None:
                viewpos = img.shape[VIEW_TABLE[ii][-1]] / 2

            else:
                viewpos = viewPositions[jj]

            self.last_view_position[ii] =\
                img.shape[VIEW_TABLE[ii][-1]] - viewpos - 1

        self.actual_slice = self.last_view_position[self.actual_view]

        # set contours
        self.contours = contours
        if self.contours is None:
            self.contours_aview = None
        else:
            self.contours_aview = self.contours.transpose(self.act_transposition)

        # masked data - has information about which data were removed
        # 1 == enabled, 0 == deleted
        # How to return:
        #       editorDialog.exec_()
        #       masked_data = editorDialog.masked
        self.masked = np.ones(self.img.shape, np.int8)

        self.voxel_size = np.squeeze(np.asarray(voxelSize))
        self.voxel_scale = self.voxel_size / float(np.min(self.voxel_size))
        self.voxel_volume = np.prod(voxelSize)

        # set seeds
        if seeds is None:
            self.seeds = np.zeros(self.img.shape, np.int8)

        else:
            self.seeds = seeds

        self.seeds_aview = self.seeds.transpose(self.act_transposition)
        self.seeds_modified = False

        self.initUI(self.img_aview.shape,
                    self.voxel_scale[np.array(self.act_transposition)],
                    600, mode)

        if mode == 'draw':
            self.seeds_orig = self.seeds.copy()
            self.slice_box.setEraseFun(self.eraseVolume)

        # set view window values C/W
        lb = np.min(img)
        self.img_min_val = lb
        ub = np.max(img)
        dul = np.double(ub) - np.double(lb)
        self.cw_range = {'c': [lb, ub], 'w': [1, dul]}
        self.slider_cw['c'].setRange(lb, ub)
        self.slider_cw['w'].setRange(1, dul)
        self.changeC(lb + dul / 2)
        self.changeW(dul)

        self.offset = np.zeros((3,), dtype=np.int16)

    def showStatus(self, msg):
        self.status_bar.showMessage(QString(msg))
        QApplication.processEvents()

    def init_draw_mask(self, draw_mask, grid):
        mask_points = []
        mask_iconlabel = []
        for mask, label in draw_mask:
            w, h = mask.shape
            xx, yy = mask.nonzero()
            mask_points.append((xx - w/2, yy - h/2))

            img = QImage(w, h, QImage.Format_ARGB32)
            img.fill(qRgba(255, 255, 255, 0))
            for ii in range(xx.shape[0]):
                img.setPixel(xx[ii], yy[ii], qRgba(0, 0, 0, 255))

            img = img.scaled(QSize(w * grid[0], h * grid[1]))
            icon = QIcon(QPixmap.fromImage(img))
            mask_iconlabel.append((icon, label))

        return mask_points, mask_iconlabel

    def saveSliceSeeds(self):
        aux = self.slice_box.getSliceSeeds()
        if aux is not None:
            self.seeds_aview[...,self.actual_slice] = aux
            self.seeds_modified = True

        else:
            self.seeds_modified = False

    def updateMaskRegion_btn(self):
        self.saveSliceSeeds()
        self.updateMaskRegion()

    def updateMaskRegion(self):
        crp = self.getCropBounds(return_nzs=True)
        if crp is not None:
            off, cri, nzs = crp
            if nzs[0].shape[0] <=5:
                self.showStatus("Not enough points (need >= 5)!")

            else:
                points = np.transpose(nzs)
                hull = Delaunay(points)
                X, Y, Z = np.mgrid[cri[0], cri[1], cri[2]]
                grid = np.vstack([X.ravel(), Y.ravel(), Z.ravel()]).T
                simplex = hull.find_simplex(grid)
                fill = grid[simplex >=0,:]
                fill = (fill[:,0], fill[:,1], fill[:,2])
                if self.contours is None or self.contours_old is None:
                    self.contours = np.zeros(self.img.shape, np.int8)
                    self.contours_old = self.contours.copy()
                else:
                    self.contours[self.contours != 2] = 0
                self.contours[fill] = 1
                self.contours_aview = self.contours.transpose(self.act_transposition)

                self.selectSlice(self.actual_slice)

    def maskRegion(self):
        self.masked[self.contours == 0] = 0

        self.img[self.contours != 2] = self.img_min_val
        self.contours.fill(0)
        self.contours_old = self.contours.copy()
        self.seeds.fill(0)
        self.selectSlice(self.actual_slice)

    def maskAddSelection(self):
        self.updateMaskRegion()
        if self.contours is None:
            return

        self.contours[self.contours == 1] = 2
        self.contours_old = self.contours.copy()
        self.seeds.fill(0)
        self.selectSlice(self.actual_slice)

    def maskRemoveSelection(self):
        self.updateMaskRegion()
        if self.contours is None:
            return

        self.contours[self.contours == 1] = 0
        self.contours_old = self.contours.copy()
        self.seeds.fill(0)
        self.selectSlice(self.actual_slice)

    def maskSelectAll(self):
        self.updateMaskRegion()

        self.seeds[0][0][0] = 1
        self.seeds[0][0][-1] = 1
        self.seeds[0][-1][0] = 1
        self.seeds[0][-1][-1] = 1
        self.seeds[-1][0][0] = 1
        self.seeds[-1][0][-1] = 1
        self.seeds[-1][-1][0] = 1
        self.seeds[-1][-1][-1] = 1

        self.updateMaskRegion()
        self.selectSlice(self.actual_slice)

    def resetSelection(self):
        self.updateMaskRegion()
        if self.contours is None:
            return

        self.contours.fill(0)
        self.contours_old = self.contours.copy()
        self.seeds.fill(0)
        self.selectSlice(self.actual_slice)

    def resetSeads(self):
        self.seeds.fill(0)
        if self.contours is not None:
            self.contours = self.contours_old.copy()
            self.contours_aview = self.contours.transpose(self.act_transposition)
        self.updateMaskRegion()
        self.selectSlice(self.actual_slice)

    def updateCropBounds(self):
        crp = self.getCropBounds()
        if crp is not None:
            _, cri = crp
            self.contours = np.zeros(self.img.shape, np.int8)
            self.contours[cri].fill(1)
            self.contours_aview = self.contours.transpose(self.act_transposition)

    def focusSliceSlider(self):
        self.slider.setFocus(True)

    def sliderSelectSlice(self, value):
        self.selectSlice(self.n_slices - value)

    def scrollSlices(self, inc):
        if abs(inc) > 0:
            new = self.actual_slice + inc
            self.selectSlice(new)

    def selectSlice(self, value, force=False):
        if not(self.allow_select_slice):
            return

        if (value < 0) or (value >= self.n_slices):
            return

        if (value != self.actual_slice) or force:
            self.saveSliceSeeds()
            if self.seeds_modified:
                if self.mode == 'crop':
                    self.updateCropBounds()

                elif self.mode == 'mask':
                    self.updateMaskRegion()

        if self.contours is None:
            contours = None

        else:
            contours = self.contours_aview[...,value]

        slider_val = self.n_slices - value
        self.slider.setValue(slider_val)
        self.slider.label.setText('Slice: %d / %d' % (slider_val, self.n_slices))

        self.slice_box.setSlice(self.img_aview[...,value],
                                self.seeds_aview[...,value],
                                contours)
        self.actual_slice = value

    def getSeeds(self):
        return self.seeds

    def getImg(self):
        return self.img

    def getOffset(self):
        return self.offset * self.voxel_size

    def getSeedsVal(self, label):
        return self.img[self.seeds==label]

    def getContours(self):
        return self.contours

    def setContours(self, contours):
        self.contours = contours
        self.contours_aview = self.contours.transpose(self.act_transposition)

        self.selectSlice(self.actual_slice)

    def changeCW(self, value, key):
        rg = self.cw_range[key]
        if (value < rg[0]) or (value > rg[1]):
            return

        if (value != self.slice_box.getCW()[key]):
            self.slider_cw[key].setValue(value)
            self.slider_cw[key].label.setText('%s: %d' % (key.upper(), value))
            self.slice_box.setCW(value, key)
            self.slice_box.updateSliceCW(self.img_aview[...,self.actual_slice])


    def changeC(self, value):
        self.changeCW(value, 'c')

    def changeW(self, value):
        self.changeCW(value, 'w')

    def setView(self, value):
        self.last_view_position[self.actual_view] = self.actual_slice
        # save seeds
        self.saveSliceSeeds()
        if self.seeds_modified:
            if self.mode == 'crop':
                self.updateCropBounds()

            elif self.mode == 'mask':
                self.updateMaskRegion()

        key = str(value)
        self.actual_view = key
        self.actual_slice = self.last_view_position[key]

        self.act_transposition = VIEW_TABLE[key]
        self.img_aview = self.img.transpose(self.act_transposition)
        self.seeds_aview = self.seeds.transpose(self.act_transposition)

        if self.contours is not None:
            self.contours_aview = self.contours.transpose(self.act_transposition)
            contours = self.contours_aview[...,self.actual_slice]

        else:
            contours = None

        vscale = self.voxel_scale[np.array(self.act_transposition)]
        height = self.slice_box.height()
        grid = height / float(self.img_aview.shape[1] * vscale[1])
        # width = (self.img_aview.shape[0] * vscale[0])[0]
        # if width > 800:
        #     height = 400
        #     grid = height / float(self.img_aview.shape[1] * vscale[1])

        mgrid = (grid * vscale[0], grid * vscale[1])

        self.slice_box.resizeSlice(new_slice_size=self.img_aview.shape[:-1],
                                   new_grid=mgrid)

        self.slice_box.setSlice(self.img_aview[...,self.actual_slice],
                                self.seeds_aview[...,self.actual_slice],
                                contours)

        self.allow_select_slice = False
        self.n_slices = self.img_aview.shape[2]
        slider_val = self.n_slices - self.actual_slice
        self.slider.setRange(1, self.n_slices)
        self.slider.setValue(slider_val)
        self.allow_select_slice = True

        self.slider.label.setText('Slice: %d / %d' % (slider_val,
                                                      self.n_slices))
        self.view_label.setText('View size: %d x %d' % self.img_aview.shape[:-1])

        self.adjustSize()
        self.adjustSize()

    def changeMask(self, val):
        self.slice_box.setMaskPoints(self.mask_points_tab[val])

    def changeContourMode(self, val):
        self.slice_box.contour_mode = str(val)
        self.slice_box.updateSlice()

    def changeEraseMode(self, val):
        self.slice_box.erase_mode = str(val)

    def eraseVolume(self, pos, mode):
        self.showStatus("Processing...")
        xyz = np.array(pos + (self.actual_slice,))
        p = np.zeros_like(xyz)
        p[np.array(self.act_transposition)] = xyz
        p = tuple(p)

        if self.seeds[p] > 0:
            if mode == 'inside':
                erase_reg(self.seeds, p, val=0)

            elif mode == 'outside':
                erase_reg(self.seeds, p, val=-1)
                idxs = np.where(self.seeds < 0)
                self.seeds.fill(0)
                self.seeds[idxs] = 1

        if self.contours is None:
            contours = None

        else:
            contours = self.contours_aview[...,self.actual_slice]

        self.slice_box.setSlice(self.img_aview[...,self.actual_slice],
                                self.seeds_aview[...,self.actual_slice],
                                contours)

        self.showStatus("Done")

    def cropUpdate(self, img):

        for ii in VIEW_TABLE.iterkeys():
            self.last_view_position[ii] = 0
        self.actual_slice = 0

        self.img = img
        self.img_aview = self.img.transpose(self.act_transposition)

        self.contours = None
        self.contours_aview = None

        self.seeds = np.zeros(self.img.shape, np.int8)
        self.seeds_aview = self.seeds.transpose(self.act_transposition)
        self.seeds_modified = False

        vscale = self.voxel_scale[np.array(self.act_transposition)]
        height = self.slice_box.height()
        grid = height / float(self.img_aview.shape[1] * vscale[1])
        mgrid = (grid * vscale[0], grid * vscale[1])

        self.slice_box.resizeSlice(new_slice_size=self.img_aview.shape[:-1],
                                   new_grid=mgrid)

        self.slice_box.setSlice(self.img_aview[...,self.actual_slice],
                                self.seeds_aview[...,self.actual_slice],
                                None)

        self.allow_select_slice = False
        self.n_slices = self.img_aview.shape[2]
        self.slider.setValue(self.actual_slice + 1)
        self.slider.setRange(1, self.n_slices)
        self.allow_select_slice = True

        self.slider.label.setText('Slice: %d / %d' % (self.actual_slice + 1,
                                                      self.n_slices))
        self.view_label.setText('View size: %d x %d' % self.img_aview.shape[:-1])

    def getCropBounds(self, return_nzs=False, flat=False):

        nzs = self.seeds.nonzero()
        cri = []
        flag = True
        for ii in range(3):
            if nzs[ii].shape[0] == 0:
                flag = False
                break

            smin, smax = np.min(nzs[ii]), np.max(nzs[ii])
            if not(flat):
                if smin == smax:
                    flag = False
                    break

            cri.append((smin, smax))

        if flag:
            cri = np.array(cri)

            out = []
            offset = []
            for jj, ii in enumerate(cri):
                out.append(slice(ii[0], ii[1] + 1))
                offset.append(ii[0])

            if return_nzs:
                return np.array(offset), tuple(out), nzs

            else:
                return np.array(offset), tuple(out)

        else:
            return None

    def crop(self):
        self.showStatus("Processing...")

        crp = self.getCropBounds()

        if crp is not None:
            offset, cri = crp
            crop = self.img[cri]
            self.img = np.ascontiguousarray(crop)
            self.offset += offset


            self.showStatus('Done')

        else:
            self.showStatus('Region not selected!')

        self.cropUpdate(self.img)

    def recalculate(self, event):
        self.saveSliceSeeds()
        if np.abs(np.min(self.seeds) - np.max(self.seeds)) < 2:
            self.showStatus('Inner and outer regions not defined!')
            return

        self.showStatus("Processing...")
        self.mode_fun(self)
        self.selectSlice(self.actual_slice)
        self.updateVolume()
        self.showStatus("Done")

    def deleteSliceSeeds(self, event):
        self.seeds_aview[...,self.actual_slice] = 0
        self.slice_box.setSlice(seeds=self.seeds_aview[...,self.actual_slice])
        self.slice_box.updateSlice()

    def resetSliceDraw(self, event):
        seeds_orig_aview = self.seeds_orig.transpose(self.act_transposition)
        self.seeds_aview[...,self.actual_slice] = seeds_orig_aview[...,self.actual_slice]
        self.slice_box.setSlice(seeds=self.seeds_aview[...,self.actual_slice])
        self.slice_box.updateSlice()

    def quit(self, event):
        self.close()

    def updateVolume(self):
        text = 'Volume:\n  unknown'
        if self.voxel_volume is not None:
            if self.mode == 'draw':
                vd = self.seeds

            else:
                vd = self.contours

            if vd is not None:
                nzs = vd.nonzero()
                nn = nzs[0].shape[0]
                if self.volume_unit == 'ml':
                    text = 'Volume [ml]:\n  %.2f' %\
                        (nn * self.voxel_volume / 1000)
                else:
                    text = 'Volume [mm3]:\n  %.2e' % (nn * self.voxel_volume)

        self.volume_label.setText(text)

    def getROI(self):
        crp = self.getCropBounds()
        if crp is not None:
            _, cri = crp

        else:
            cri = []
            for jj, ii in enumerate(self.img.shape):
                off = self.offset[jj]
                cri.append(slice(off, off + ii))

        return cri

def gen_test():
    test = {}
    test['data'] = np.zeros((10,10,4), dtype=np.uint8)
    test['voxelsize_mm'] = (2, 2, 2.5)

    return test

usage = '%prog [options]\n' + __doc__.rstrip()
help = {
    'in_file': 'input *.mat file with "data" field',
    'mode': '"seed", "crop", "mask" or "draw" mode',
    #'out_file': 'store the output matrix to the file',
    #'debug': 'run in debug mode',
    'gen_test': 'generate test data',
    'test': 'run unit test',
}

def main():
    parser = OptionParser(description='Segmentation editor')
    parser.add_option('-f','--filename', action='store',
                      dest='in_filename', default=None,
                      help=help['in_file'])
    # parser.add_option('-d', '--debug', action='store_true',
    #                   dest='debug', help=help['debug'])
    parser.add_option('-m', '--mode', action='store',
                      dest='mode', default='seed', help=help['mode'])
    parser.add_option('-t', '--tests', action='store_true',
                      dest='unit_test', help=help['test'])
    parser.add_option('-g', '--gener_data', action='store_true',
                      dest='gen_test', help=help['gen_test'])

    # parser.add_option('-o', '--outputfile', action='store',
    #                   dest='out_filename', default='output.mat',
    #                   help=help['out_file'])
    (options, args) = parser.parse_args()

    if options.gen_test:
        dataraw = gen_test()

    else:
        if options.in_filename is None:
            raise IOError('No input data!')

        else:
            dataraw = loadmat(options.in_filename,
                              variable_names=['data', 'segdata',
                                              'voxelsize_mm', 'seeds'])
            if not ('segdata' in dataraw):
                dataraw['segdata'] = None

    app = QApplication(sys.argv)
    pyed = QTSeedEditor(dataraw['data'],
                        seeds=dataraw['segdata'],
                        mode=options.mode,
                        voxelSize=dataraw['voxelsize_mm'])
    sys.exit(app.exec_())

if __name__ == "__main__":
    # main()

    app = QApplication(sys.argv)
    data_s = 200 * np.triu(np.ones((6,8), dtype=np.int))
    data = np.dstack((data_s, data_s))
    data = np.rollaxis(data, 2, 0)
    pyed = QTSeedEditor(data)

    sys.exit(app.exec_())


__author__ = 'tomas'


import sys

from PyQt4 import QtGui, QtCore
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt4agg import NavigationToolbar2QTAgg as NavigationToolbar

import matplotlib.pyplot as plt

import random
import numpy as np


class TumorVisualiser(QtGui.QMainWindow):

    def __init__(self, im, labels, healthy_label, hypo_label, hyper_label, disp_smoothed=False):
        super(TumorVisualiser, self).__init__()

        self.im = im
        self.labels = labels
        self.show_view_1 = True
        self.show_view_2 = True
        self.healthy_label = healthy_label
        self.hypo_label = hypo_label
        self.hyper_label = hyper_label
        self.disp_smoothed = disp_smoothed
        # self.data_1 = self.im
        # self.data_2 = self.labels

        self.init_UI_win()

    def init_UI_win(self):

        # TOOLBARS ----------------------
        # # menu
        # exitAction = QtGui.QAction(QtGui.QIcon('icons/Exit.png'), 'menu', self)
        # # exitAction.setShortcut('Ctrl+Q')
        # exitAction.triggered.connect(QtGui.qApp.quit)
        # self.toolbar_menu = self.addToolBar('menu')
        # self.toolbar_menu.addAction(exitAction)

        # view 1
        view_1_action = QtGui.QAction(QtGui.QIcon('icons/Eye.png'), 'view1', self)
        view_1_action.triggered.connect(self.view_1_callback)
        self.toolbar_view_1 = self.addToolBar('view1')
        self.toolbar_view_1.addAction(view_1_action)

        #view 2
        view_2_action = QtGui.QAction(QtGui.QIcon('icons/Eye.png'), 'view2', self)
        view_2_action.triggered.connect(self.view_2_callback)
        self.toolbar_view_2 = self.addToolBar('view2')
        self.toolbar_view_2.addAction(view_2_action)

        # show input data on view 1
        self.show_im_1_action = QtGui.QAction(QtGui.QIcon('icons/Stock graph.png'), 'im_1', self)
        self.show_im_1_action.triggered.connect(self.show_im_1_callback)
        self.toolbar_view_1.addAction(self.show_im_1_action)

        # show input data on view 2
        self.show_im_2_action = QtGui.QAction(QtGui.QIcon('icons/Stock graph.png'), 'im_2', self)
        self.show_im_2_action.triggered.connect(self.show_im_2_callback)
        self.toolbar_view_2.addAction(self.show_im_2_action)

        # show label data on view 1
        self.show_labels_1_action = QtGui.QAction(QtGui.QIcon('icons/Blue tag.png'), 'labels_1', self)
        self.show_labels_1_action.triggered.connect(self.show_labels_1_callback)
        self.toolbar_view_1.addAction(self.show_labels_1_action)

        # show label data on view 2
        self.show_labels_2_action = QtGui.QAction(QtGui.QIcon('icons/Blue tag.png'), 'labels_2', self)
        self.show_labels_2_action.triggered.connect(self.show_labels_2_callback)
        self.toolbar_view_2.addAction(self.show_labels_2_action)

        # show contours data on view 1
        self.show_contours_1_action = QtGui.QAction(QtGui.QIcon('icons/Brush.png'), 'contours_1', self)
        self.show_contours_1_action.triggered.connect(self.show_contours_1_callback)
        self.toolbar_view_1.addAction(self.show_contours_1_action)

        # show contours data on view 2
        self.show_contours_2_action = QtGui.QAction(QtGui.QIcon('icons/Brush.png'), 'contours_2', self)
        self.show_contours_2_action.triggered.connect(self.show_contours_2_callback)
        self.toolbar_view_2.addAction(self.show_contours_2_action)


        # STATUS BAR -------------------
        # self.status_bar = QtGui.QStatusBar()
        self.statusBar()

        # MAIN WINDOW -------------------
        # self.setGeometry(300, 300, 300, 200)
        self.center()
        self.setWindowTitle('Tumor Visualiser')

        self.form_widget = FormWidget(self)

        self.setCentralWidget(self.form_widget)

    def view_1_callback(self):
        self.show_view_1 = not self.show_view_1

        # enabling and disabling other toolbar icons
        self.show_im_1_action.setEnabled(not self.show_im_1_action.isEnabled())
        self.show_labels_1_action.setEnabled(not self.show_labels_1_action.isEnabled())
        self.show_contours_1_action.setEnabled(not self.show_contours_1_action.isEnabled())

        self.statusBar().showMessage('view_1 set to %s' % self.show_view_1)
        # print 'view_1 set to', self.show_view_1
        self.form_widget.update_figures()

    def view_2_callback(self):
        self.show_view_2 = not self.show_view_2

        # enabling and disabling other toolbar icons
        self.show_im_2_action.setEnabled(not self.show_im_2_action.isEnabled())
        self.show_labels_2_action.setEnabled(not self.show_labels_2_action.isEnabled())
        self.show_contours_2_action.setEnabled(not self.show_contours_2_action.isEnabled())

        self.statusBar().showMessage('view_2 set to %s' % self.show_view_2)
        # print 'view_2 set to', self.show_view_2
        self.form_widget.update_figures()

    def show_im_1_callback(self):
        # print 'data_1 set to im'
        self.statusBar().showMessage('data_1 set to im')
        self.form_widget.data_1 = self.im
        self.form_widget.data_1_str = 'im'
        self.form_widget.update_figures()

    def show_im_2_callback(self):
        # print 'data_2 set to im'
        self.statusBar().showMessage('data_2 set to im')
        if self.disp_smoothed:
            self.form_widget.data_2 = self.labels
        else:
            self.form_widget.data_2 = self.im
        self.form_widget.data_2_str = 'im'
        self.form_widget.update_figures()

    def show_labels_1_callback(self):
        # print 'data_1 set to labels'
        self.statusBar().showMessage('data_1 set to labels')
        self.form_widget.data_1 = self.labels
        self.form_widget.data_1_str = 'labels'
        self.form_widget.update_figures()

    def show_labels_2_callback(self):
        # print 'data_2 set to labels'
        self.statusBar().showMessage('data_2 set to labels')
        self.form_widget.data_2 = self.labels
        self.form_widget.data_2_str = 'labels'
        self.form_widget.update_figures()

    def show_contours_1_callback(self):
        # print 'data_2 set to contours'
        self.statusBar().showMessage('data_1 set to contours')
        self.form_widget.data_1 = self.im
        self.form_widget.data_1_str = 'contours'
        self.form_widget.update_figures()

    def show_contours_2_callback(self):
        # print 'data_2 set to contours'
        self.statusBar().showMessage('data_2 set to contours')
        self.form_widget.data_2 = self.im
        self.form_widget.data_2_str = 'contours'
        self.form_widget.update_figures()

    def center(self):
        qr = self.frameGeometry()
        cp = QtGui.QDesktopWidget().availableGeometry().center()
        qr.moveCenter(cp)
        self.move(qr.topLeft())

# Main widget containing figures etc
class FormWidget(QtGui.QWidget):

    def __init__(self, window):

        self.win = window  # link to the main window
        self.im = self.win.im  # input data
        self.labels = self.win.labels  # input labeling
        self.actual_slice = 0  # index of current data slice
        self.n_slices = self.im.shape[2]  # numer of slices
        self.healthy_label = self.win.healthy_label
        self.hypo_label = self.win.hypo_label
        self.hyper_label = self.win.hyper_label

        super(FormWidget, self).__init__()
        self.init_UI_form()

    def init_UI_form(self):

        # QtGui.QToolTip.setFont(QtGui.QFont('SansSerif', 10))

        self.data_1 = self.im  # data to be shown in view_1
        self.data_2 = self.labels  # data to be shown in view_2
        self.data_1_str = 'im'
        self.data_2_str = 'labels'

        self.figure = plt.figure()
        self.axes = self.figure.add_axes([0, 0, 1, 1])

        self.canvas = FigureCanvas(self.figure)

        # Just some button connected to `plot` method
        # self.button = QtGui.QPushButton('Plot')
        # self.button.clicked.connect(self.plot)

        # slider
        # self.slider = QtGui.QSlider(QtCore.Qt.Vertical, self)
        self.slider = QtGui.QSlider(QtCore.Qt.Horizontal, self)
        self.slider.setMinimum(0)
        self.slider.setMaximum(self.n_slices - 1)
        # self.slider.setMaximum(10)
        self.slider.valueChanged[int].connect(self.slider_change)
        self.slider.setSingleStep(1)  # step for arrows
        self.slider.setPageStep(1)  # step for mouse wheel

        # set the layout
        self.slice_label = QtGui.QLabel('slice #: %i/%i' % (self.actual_slice + 1, self.n_slices))
        slider_layout = QtGui.QHBoxLayout()
        # self.slice_label = QtGui.QLabel('slice #:\n%i/%i' % (self.actual_slice + 1, self.n_slices))
        # slider_layout = QtGui.QVBoxLayout()
        slider_layout.addWidget(self.slice_label)
        slider_layout.addWidget(self.slider)
        slider_frame = QtGui.QFrame()
        slider_frame.setLayout(slider_layout)

        # layout = QtGui.QHBoxLayout()
        layout = QtGui.QVBoxLayout()
        layout.addWidget(self.canvas, 1)
        layout.addWidget(slider_frame)
        # layout.addWidget(self.button)
        self.setLayout(layout)

        # # BUTTONS -----------------------
        # btn = QtGui.QPushButton('Button', self)
        # btn.setToolTip('This is a <b>QPushButton</b> widget')
        # btn.resize(btn.sizeHint())
        # btn.move(50, 50)

        # LAYOUT ------------------------
        # main_hbox = QtGui.QHBoxLayout()
        # main_hbox.addStretch(1)
        # main_hbox.addWidget(self.toolbar_menu)
        #
        # view_vbox = QtGui.QVBoxLayout()
        # view_vbox.addStretch(1)
        # view_vbox.addWidget(self.toolbar_view_1)
        # view_vbox.addWidget(self.toolbar_view_2)

        # conenction to wheel events
        self.canvas.mpl_connect('scroll_event', self.on_scroll)

        self.update_figures()

    def slider_change(self, value):
        # self.win.status_bar.showMessage('slider changed to %i' % value)
        # self.win.statusBar().showMessage('actual slice changed to %i' % value)
        self.actual_slice = value
        self.update_slice_label()
        self.update_figures()

    def label2rgb(self, slice):
        r = slice == self.hyper_label
        g = slice == self.healthy_label
        b = slice == self.hypo_label
        slice_rgb = np.dstack((r, g, b))

        return slice_rgb

    def update_figures(self):
        # setting the minimal and maximal to scale luminance data
        if self.data_1_str is 'labels':
            vmin1 = self.data_1.min()
            vmax1 = self.data_1.max()
            slice_1 = self.label2rgb(self.data_1[:, :, self.actual_slice])
        else:
            vmin1 = 0
            vmax1 = 255
            slice_1 = self.data_1[:, :, self.actual_slice]
        if self.data_2_str is 'labels':
            vmin2 = self.data_2.min()
            vmax2 = self.data_2.max()
            slice_2 = self.label2rgb(self.data_2[:, :, self.actual_slice])
        else:
            vmin2 = 0
            vmax2 = 255
            if self.win.disp_smoothed:
                vmax2 = self.labels.max()
            slice_2 = self.data_2[:, :, self.actual_slice]

        # if both views are enabled
        if self.win.show_view_1 and self.win.show_view_2:
            plt.figure(self.figure.number)
            plt.subplot(121)
            self.figure.gca().cla()  # clearing the contours, just to be sure
            plt.imshow(slice_1, 'gray', interpolation='nearest', vmin=vmin1, vmax=vmax1)
            # displaying contours if desirable
            if self.data_1_str is 'contours':
                self.draw_contours()
            plt.title('view_1: %s' % self.data_1_str)

            plt.subplot(122)
            self.figure.gca().cla()  # clearing the contours, just to be sure
            plt.imshow(slice_2, 'gray', interpolation='nearest', vmin=vmin2, vmax=vmax2)
            # displaying contours if desirable
            if self.data_2_str is 'contours':
                self.draw_contours()
            plt.title('view_2: %s' % self.data_2_str)

        # if only the first view is enabled
        elif self.win.show_view_1:
            plt.figure(self.figure.number)
            plt.subplot(111)
            self.figure.gca().cla()  # clearing the contours, just to be sure
            plt.imshow(slice_1, 'gray', interpolation='nearest', vmin=vmin1, vmax=vmax1)
            if self.data_1_str is 'contours':
                self.draw_contours()
            plt.title('view_1: %s' % self.data_1_str)

        # if only the second view is enabled
        elif self.win.show_view_2:
            plt.figure(self.figure.number)
            plt.subplot(111)
            self.figure.gca().cla()  # clearing the contours, just to be sure
            plt.imshow(slice_2, 'gray', interpolation='nearest', vmin=vmin2, vmax=vmax2)
            if self.data_2_str is 'contours':
                self.draw_contours()
            plt.title('view_2: %s' % self.data_2_str)
        else:
            plt.figure(self.figure.number)
            plt.clf()

        self.canvas.draw()

    def draw_contours(self):
        try:
            self.figure.gca().contour(self.labels[:, :, self.actual_slice] == self.healthy_label, [0.5], colors='g', linewidths=2)
        except:
            print 'contour fail: ', sys.exc_info()[0]
        try:
            self.figure.gca().contour(self.labels[:, :, self.actual_slice] == self.hypo_label, [0.5], colors='b', linewidths=2)
        except:
            print 'contour fail: ', sys.exc_info()[0]
        try:
            self.figure.gca().contour(self.labels[:, :, self.actual_slice] == self.hyper_label, [0.5], colors='r', linewidths=2)
        except:
            print 'contour fail: ', sys.exc_info()[0]

    def update_slice_label(self):
        self.slice_label.setText('slice #: %i/%i' % (self.actual_slice + 1, self.n_slices))

    def next_slice(self):
        self.actual_slice += 1
        if self.actual_slice >= self.n_slices:
            self.actual_slice = 0

    def prev_slice(self):
        self.actual_slice -= 1
        if self.actual_slice < 0:
            self.actual_slice = self.n_slices - 1

    def on_scroll(self, event):
        '''mouse wheel is used for setting slider value'''
        if event.button == 'up':
            self.next_slice()
        if event.button == 'down':
            self.prev_slice()
        self.slider.setValue(self.actual_slice)
        # self.slider_change(self.actual_slice)


def run(im, labels, healthy_label, hypo_label, hyper_label, slice_axis=2, disp_smoothed=False):
    if slice_axis == 0:
        im = np.transpose(im, (1, 2, 0))
        labels = np.transpose(labels, (1, 2, 0))
    app = QtGui.QApplication(sys.argv)
    tv = TumorVisualiser(im, labels, healthy_label, hypo_label, hyper_label, disp_smoothed)
    tv.show()
    sys.exit(app.exec_())

def main():
    # parameters
    params = dict()
    # params['hypo_label'] = 1
    # params['hyper_label'] = 2
    healthy_label = 0
    hypo_label = 1
    hyper_label = 2

    # preparing data
    size = 100
    n_slices = 4
    im = np.zeros((size, size, n_slices))
    step = size / n_slices
    for i in range(n_slices):
        im[i * step:(i + 1) * step, :, i] = 150

    labels = np.zeros((size, size, n_slices))
    for i in range(n_slices):
        if np.mod(i, 2) == 0:
            lab = 1
        else:
            lab = 2
        labels[:, i * step:(i + 1) * step, i] = lab

    # runing application
    run(im, labels, healthy_label, hypo_label, hyper_label)

if __name__ == '__main__':
    main()


__author__ = 'Ryba'
import numpy as np
import matplotlib.pyplot as plt
import skimage.exposure as skiexp
from skimage.segmentation import mark_boundaries
import os
import glob
import dicom
import cv2
# from skimage import measure
import skimage.measure as skimea
import skimage.morphology as skimor
import skimage.transform as skitra
import skimage.filter as skifil
# import skimage.restoration as skires
import skimage.filter as skifil
import skimage.segmentation as skiseg
import scipy.stats as scista
import scipy.ndimage.morphology as scindimor
import scipy.ndimage.measurements as scindimea
import scipy.ndimage.interpolation as scindiint

import pickle

import py3DSeedEditor

#----------------------------------------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------------------------------------
def get_seeds(im, minT=0.95, maxT=1.05, minInt=0, maxInt=255, debug=False):
    vals = im[np.where(np.logical_and(im>=minInt, im<=maxInt))]
    hist, bins = skiexp.histogram(vals)
    max_peakIdx = hist.argmax()

    minT *= bins[max_peakIdx]
    maxT *= bins[max_peakIdx]
    histTIdxs = (bins >= minT) * (bins <= maxT)
    histTIdxs = np.nonzero(histTIdxs)[0]
    class1TMin = minT
    class1TMax = maxT

    seed_mask = np.where( (im >= class1TMin) * (im <= class1TMax), 1, 0)

    if debug:
        plt.figure()
        plt.plot(bins, hist)
        plt.hold(True)

        plt.plot(bins[max_peakIdx], hist[max_peakIdx], 'ro')
        plt.plot(bins[histTIdxs], hist[histTIdxs], 'r')
        plt.plot(bins[histTIdxs[0]], hist[histTIdxs[0]], 'rx')
        plt.plot(bins[histTIdxs[-1]], hist[histTIdxs[-1]], 'rx')
        plt.title('Image histogram and its class1 = maximal peak (red dot) +/- minT/maxT % of its density (red lines).')
        plt.show()

    #minT *= hist[max_peakIdx]
    #maxT *= hist[max_peakIdx]
    #histTIdxs = (hist >= minT) * (hist <= maxT)
    #histTIdxs = np.nonzero(histTIdxs)[0]
    #histTIdxs = histTIdxs.astype(np.int)minT *= hist[max_peakIdx]
    #class1TMin = bins[histTIdxs[0]]
    #class1TMax = bins[histTIdxs[-1]

    #if debug:
    #    plt.figure()
    #    plt.plot(bins, hist)
    #    plt.hold(True)
    #
    #    plt.plot(bins[max_peakIdx], hist[max_peakIdx], 'ro')
    #    plt.plot(bins[histTIdxs], hist[histTIdxs], 'r')
    #    plt.plot(bins[histTIdxs[0]], hist[histTIdxs[0]], 'rx')
    #    plt.plot(bins[histTIdxs[-1]], hist[histTIdxs[-1]], 'rx')
    #    plt.title('Image histogram and its class1 = maximal peak (red dot) +/- minT/maxT % of its density (red lines).')
    #    plt.show()

    return seed_mask, class1TMin, class1TMax


#----------------------------------------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------------------------------------
def seeds2superpixels(seed_mask, superpixels, debug=False, im=None):
    seeds = np.argwhere(seed_mask)
    superseeds = np.zeros_like(seed_mask)

    for s in seeds:
        label = superpixels[s[0], s[1]]
        superseeds = np.where(superpixels==label, 1, superseeds)

    if debug:
        plt.figure(), plt.gray()
        plt.subplot(121), plt.imshow(im), plt.hold(True), plt.plot(seeds[:,1], seeds[:,0], 'ro'), plt.axis('image')
        plt.subplot(122), plt.imshow(im), plt.hold(True), plt.plot(seeds[:,1], seeds[:,0], 'ro'),
        plt.imshow(mark_boundaries(im, superseeds, color=(1,0,0))), plt.axis('image')
        plt.show()

    return superseeds


#----------------------------------------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------------------------------------
def intensity_range2superpixels(im, superpixels, intMinT=0.95, intMaxT=1.05, debug=False, intMin=0, intMax=255):#, fromInt=0, toInt=255):

    superseeds = np.zeros_like(superpixels)

    #if not intMin and not intMax:
    #    hist, bins = skexp.histogram(im)
    #
    #    #zeroing values that are lower/higher than fromInt/toInt
    #    toLow = np.where(bins < fromInt)
    #    hist[toLow] = 0
    #    toHigh = np.where(bins > toInt)
    #    hist[toHigh] = 0
    #
    #    max_peakIdx = hist.argmax()
    #    intMin = intMinT * bins[max_peakIdx]
    #    intMax = intMaxT * bins[max_peakIdx]

    sp_means = np.zeros(superpixels.max()+1)
    for sp in range(superpixels.max()+1):
        values = im[np.where(superpixels==sp)]
        mean = np.mean(values)
        sp_means[sp] = mean

    idxs = np.argwhere(np.logical_and(sp_means>=intMin, sp_means<=intMax))
    for i in idxs:
        superseeds = np.where(superpixels==i[0], 1, superseeds)

    if debug:
        plt.figure(), plt.gray()
        plt.imshow(im), plt.hold(True), plt.imshow(mark_boundaries(im, superseeds, color=(1,0,0)))
        plt.axis('image')
        plt.show()

    return superseeds


def show_slice(data, segmentation=None, lesions=None, show='True'):
    plt.figure()
    plt.gray()
    plt.imshow(data)

    if segmentation is not None:
        plt.hold(True)
        contours = skimea.find_contours(segmentation, 1)
        for contour in contours:
            plt.plot(contour[:, 1], contour[:, 0], 'b', linewidth=2)

    if lesions is not None:
        plt.hold(True)
        contours = skimea.find_contours(lesions, 1)
        for contour in contours:
            plt.plot(contour[:, 1], contour[:, 0], 'r', linewidth=2)

    plt.axis('image')

    if show:
        plt.show()


def change_slice_index(data):
    n_slices = data.shape[2]
    data_reshaped = np.zeros(np.hstack((data.shape[2], data.shape[0], data.shape[1])))
    for i in range(n_slices):
        data_reshaped[i, :, :] = data[:, :, i]
    return data_reshaped


def read_data(dcmdir, indices=None, wildcard='*.dcm', type=np.int16):

    dcmlist = []
    for infile in glob.glob(os.path.join(dcmdir, wildcard)):
        dcmlist.append(infile)

    if indices == None:
        indices = range(len(dcmlist))

    data3d = []
    for i in range(len(indices)):
        ind = indices[i]
        onefile = dcmlist[ind]
        if wildcard == '*.dcm':
            data = dicom.read_file(onefile)
            data2d = data.pixel_array
            try:
                data2d = (np.float(data.RescaleSlope) * data2d) + np.float(data.RescaleIntercept)
            except:
                print('problem with RescaleSlope and RescaleIntercept')
        else:
            data2d = cv2.imread(onefile, 0)

        if len(data3d) == 0:
            shp2 = data2d.shape
            data3d = np.zeros([shp2[0], shp2[1], len(indices)], dtype=type)

        data3d[:,:,i] = data2d

    #need to reshape data to have slice index (ndim==3)
    if data3d.ndim == 2:
        data3d.resize(np.hstack((data3d.shape,1)))

    return data3d


def windowing(data, level=50, width=300, sub1024=False, sliceId=2):
    #srovnani na standardni skalu = odecteni 1024HU
    if sub1024:
        data -= 1024

    #zjisteni minimalni a maximalni density
    minHU = level - width
    maxHU = level + width

    if data.ndim == 3:
        if sliceId == 2:
            for idx in range(data.shape[2]):
                #rescalovani intenzity tak, aby skala <minHU, maxHU> odpovidala intervalu <0,255>
                data[:, :, idx] = skiexp.rescale_intensity(data[:, :, idx], in_range=(minHU, maxHU), out_range=(0, 255))
        elif sliceId == 0:
            for idx in range(data.shape[0]):
                #rescalovani intenzity tak, aby skala <minHU, maxHU> odpovidala intervalu <0,255>
                data[idx, :, :] = skiexp.rescale_intensity(data[idx, :, :], in_range=(minHU, maxHU), out_range=(0, 255))
    else:
        data = skiexp.rescale_intensity(data, in_range=(minHU, maxHU), out_range=(0, 255))

    return data.astype(np.uint8)


def smoothing(data, d=10, sigmaColor=10, sigmaSpace=10, sliceId=2):
    if data.ndim == 3:
        if sliceId == 2:
            for idx in range(data.shape[2]):
                data[:, :, idx] = cv2.bilateralFilter( data[:, :, idx], d=d, sigmaColor=sigmaColor, sigmaSpace=sigmaSpace )
        elif sliceId == 0:
            for idx in range(data.shape[0]):
                data[idx, :, :] = cv2.bilateralFilter( data[idx, :, :], d=d, sigmaColor=sigmaColor, sigmaSpace=sigmaSpace )
    else:
        data = cv2.bilateralFilter( data, d=d, sigmaColor=sigmaColor, sigmaSpace=sigmaSpace )
    return data


def smoothing_bilateral(data, sigma_space=15, sigma_color=0.05, pseudo_3D='True', sliceId=2):
    if data.ndim == 3 and pseudo_3D:
        if sliceId == 2:
            for idx in range(data.shape[2]):
                temp = skifil.denoise_bilateral(data[:, :, idx], sigma_range=sigma_color, sigma_spatial=sigma_space)
                # temp = skires.denoise_bilateral(data[:, :, idx], sigma_range=sigma_color, sigma_spatial=sigma_space)
                data[:, :, idx] = (255 * temp).astype(np.uint8)
        elif sliceId == 0:
            for idx in range(data.shape[0]):
                temp = skifil.denoise_bilateral(data[idx, :, :], sigma_range=sigma_color, sigma_spatial=sigma_space)
                # temp = skires.denoise_bilateral(data[idx, :, :], sigma_range=sigma_color, sigma_spatial=sigma_space)
                data[idx, :, :] = (255 * temp).astype(np.uint8)
    else:
        data = skifil.denoise_bilateral(data, sigma_range=sigma_color, sigma_spatial=sigma_space)
        # data = skires.denoise_bilateral(data, sigma_range=sigma_color, sigma_spatial=sigma_space)
        data = (255 * data).astype(np.uint8)
    return data


def smoothing_tv(data, weight=0.1, pseudo_3D=True, multichannel=False, sliceId=2):
    if data.ndim == 3 and pseudo_3D:
        if sliceId == 2:
            for idx in range(data.shape[2]):
                temp = skifil.denoise_tv_chambolle(data[:, :, idx], weight=weight, multichannel=multichannel)
                # temp = skires.denoise_tv_chambolle(data[:, :, idx], weight=weight, multichannel=multichannel)
                data[:, :, idx] = (255 * temp).astype(np.uint8)
        elif sliceId == 0:
            for idx in range(data.shape[0]):
                temp = skifil.denoise_tv_chambolle(data[idx, :, :], weight=weight, multichannel=multichannel)
                # temp = skires.denoise_tv_chambolle(data[idx, :, :], weight=weight, multichannel=multichannel)
                data[idx, :, :] = (255 * temp).astype(np.uint8)
    else:
        data = skifil.denoise_tv_chambolle(data, weight=weight, multichannel=False)
        # data = skires.denoise_tv_chambolle(data, weight=weight, multichannel=False)
        data = (255 * data).astype(np.uint8)
    return data


def smoothing_gauss(data, sigma=1, pseudo_3D='True', sliceId=2):
    if data.ndim == 3 and pseudo_3D:
        if sliceId == 2:
            for idx in range(data.shape[2]):
                temp = skifil.gaussian_filter(data[:, :, idx], sigma=sigma)
                data[:, :, idx] = (255 * temp).astype(np.uint8)
        elif sliceId == 0:
            for idx in range(data.shape[0]):
                temp = skifil.gaussian_filter(data[idx, :, :], sigma=sigma)
                data[idx, :, :] = (255 * temp).astype(np.uint8)
    else:
        data = skifil.gaussian_filter(data, sigma=sigma)
        data = (255 * data).astype(np.uint8)
    return data



def analyse_histogram(data, roi=None, debug=False, dens_min=20, dens_max=255, minT=0.95, maxT=1.05):
    if roi == None:
        #roi = np.ones(data.shape, dtype=np.bool)
        roi = np.logical_and(data >= dens_min, data <= dens_max)

    voxels = data[np.nonzero(roi)]
    hist, bins = skiexp.histogram(voxels)
    max_peakIdx = hist.argmax()

    minT = minT * hist[max_peakIdx]
    maxT = maxT * hist[max_peakIdx]
    histTIdxs = (hist >= minT) * (hist <= maxT)
    histTIdxs = np.nonzero(histTIdxs)[0]
    histTIdxs = histTIdxs.astype(np.int)

    class1TMin = bins[histTIdxs[0]]
    class1TMax = bins[histTIdxs[-1]]

    liver = data * (roi > 0)
    class1 = np.where( (liver >= class1TMin) * (liver <= class1TMax), 1, 0)

    if debug:
        plt.figure()
        plt.plot(bins, hist)
        plt.hold(True)

        plt.plot(bins[max_peakIdx], hist[max_peakIdx], 'ro')
        plt.plot(bins[histTIdxs], hist[histTIdxs], 'r')
        plt.plot(bins[histTIdxs[0]], hist[histTIdxs[0]], 'rx')
        plt.plot(bins[histTIdxs[-1]], hist[histTIdxs[-1]], 'rx')
        plt.title('Histogram of liver density and its class1 = maximal peak (red dot) +-5% of its density (red line).')
        plt.show()

    return class1


def intensity_probability(data, std=20, roi=None, dens_min=10, dens_max=255):
    if roi == None:
        # roi = np.logical_and(data >= dens_min, data <= dens_max)
        roi = np.ones(data.shape, dtype=np.bool)
    voxels = data[np.nonzero(roi)]
    hist, bins = skiexp.histogram(voxels)

    #zeroing histogram outside interval <dens_min, dens_max>
    hist[:dens_min] = 0
    hist[dens_max:] = 0

    max_id = hist.argmax()
    mu = round(bins[max_id])

    prb = scista.norm(loc=mu, scale=std)

    print('liver pdf: mu = %i, std = %i'%(mu, std))

    # plt.figure()
    # plt.plot(bins, hist)
    # plt.hold(True)
    # plt.plot(mu, hist[max_id], 'ro')
    # plt.show()

    probs_L = prb.pdf(voxels)
    probs = np.zeros(data.shape)

    coords = np.argwhere(roi)
    n_elems = coords.shape[0]
    for i in range(n_elems):
        if data.ndim == 3:
            probs[coords[i,0], coords[i,1], coords[i,2]] = probs_L[i]
        else:
            probs[coords[i,0], coords[i,1]] = probs_L[i]

    return probs


def get_zunics_compatness(obj):
    m000 = obj.sum()
    m200 = get_central_moment(obj, 2, 0, 0)
    m020 = get_central_moment(obj, 0, 2, 0)
    m002 = get_central_moment(obj, 0, 0, 2)
    term1 = (3**(5./3)) / (5 * (4*np.pi)**(2./3))
    term2 = m000**(5./3) / (m200 + m020 + m002)
    K = term1 * term2
    return K


def get_central_moment(obj, p, q, r):
    elems = np.argwhere(obj)
    m000 = obj.sum()
    m100 = (elems[:,0]).sum()
    m010 = (elems[:,1]).sum()
    m001 = (elems[:,2]).sum()
    xc = m100 / m000
    yc = m010 / m000
    zc = m001 / m000

    mom = 0
    for el in elems:
        mom += (el[0] - xc)**p + (el[1] - yc)**q + (el[2] - zc)**r

    return mom


def opening3D(data, selem=skimor.disk(3), sliceId=0):
    if sliceId == 0:
        for i in range(data.shape[0]):
            data[i,:,:] = skimor.binary_opening(data[i,:,:], selem)
    elif sliceId == 2:
        for i in range(data.shape[2]):
            data[:,:,i] = skimor.binary_opening(data[:,:,i], selem)
    return data


def closing3D(data, selem=skimor.disk(3), slicewise=False, sliceId=0):
    if slicewise:
        if sliceId == 0:
            for i in range(data.shape[0]):
                data[i, :, :] = skimor.binary_closing(data[i, :, :], selem)
        elif sliceId == 2:
            for i in range(data.shape[2]):
                data[:, :, i] = skimor.binary_closing(data[:, :, i], selem)
    else:
        data = scindimor.binary_closing(data, selem)
    return data


def eroding3D(data, selem=skimor.disk(3), slicewise=False, sliceId=0):
    if slicewise:
        if sliceId == 0:
            for i in range(data.shape[0]):
                data[i, :, :] = skimor.binary_erosion(data[i, :, :], selem)
        elif sliceId == 2:
            for i in range(data.shape[2]):
                data[:, :, i] = skimor.binary_erosion(data[:, :, i], selem)
    else:
        data = scindimor.binary_erosion(data, selem)
    return data


def resize3D(data, scale=None, shape=None,sliceId=2, method='cv2'):
    if sliceId == 2:
        n_slices = data.shape[2]
        # new_shape = cv2.resize(data[:,:,0], None, fx=scale, fy=scale).shape
        new_shape = scindiint.zoom(data[:,:,0], scale).shape
        new_data = np.zeros(np.hstack((new_shape,n_slices)), dtype=np.int)
        for i in range(n_slices):
            # new_data[:,:,i] = cv2.resize(data[:,:,i], None, fx=scale, fy=scale)
            # new_data[:,:,i] = (255 * skitra.rescale(data[:,:,0], scale)).astype(np.int)
            if method == 'cv2':
                new_data[:,:,i] = cv2.resize(data[:,:,i], (0,0),  fx=scale, fy=scale, interpolation=cv2.INTER_NEAREST)
            else:
                new_data[:,:,i] = scindiint.zoom(data[:,:,i], scale)
    elif sliceId == 0:
        n_slices = data.shape[0]
        # new_shape = cv2.resize(data[0,:,:], None, fx=scale, fy=scale).shape
        # new_shape = skitra.rescale(data[0,:,:], scale).shape
        if scale is not None:
            # new_shape = scindiint.zoom(data[0,:,:], scale).shape
            # new_shape = np.hstack((n_slices, scindiint.zoom(data[0,:,:], scale).shape))
            new_shape = np.hstack((n_slices, cv2.resize(data[0,:,:], (0,0), fx=scale, fy=scale, interpolation=cv2.INTER_NEAREST).shape))
            # new_data = np.zeros(new_shape, dtype=np.int)
        else:
            new_shape = shape
        new_data = np.zeros(new_shape, dtype=np.int)

        for i in range(n_slices):
            # new_data[i,:,:] = cv2.resize(data[i,:,:], None, fx=scale, fy=scale)
            # new_data[i,:,:] = (255 * skitra.rescale(data[i,:,:], scale)).astype(np.int)
            if method == 'cv2':
                if scale is not None:
                    new_data[i,:,:] = cv2.resize(data[i,:,:], (0,0), fx=scale, fy=scale, interpolation=cv2.INTER_NEAREST)
                elif shape is not None:
                    new_data[i,:,:] = cv2.resize(data[i,:,:], new_shape[1:][::-1], interpolation=cv2.INTER_NEAREST)
            else:
                new_data[i,:,:] = scindiint.zoom(data[i,:,:], scale)
    return new_data


def get_overlay(mask, alpha=0.3, color='r'):
    layer = None
    if color == 'r':
        layer = np.dstack((255*mask, np.zeros_like(mask), np.zeros_like(mask), alpha * mask))
    elif color == 'g':
        layer = alpha * np.dstack((np.zeros_like(mask), mask, np.zeros_like(mask)))
    elif color == 'b':
        layer = alpha * np.dstack((np.zeros_like(mask), np.zeros_like(mask), mask))
    elif color == 'c':
        layer = alpha * np.dstack((np.zeros_like(mask), mask, mask))
    elif color == 'm':
        layer = alpha * np.dstack((mask, np.zeros_like(mask), mask))
    elif color == 'y':
        layer = alpha * np.dstack((mask, mask, np.zeros_like(mask)))
    else:
        print 'Unknown color, using red as default.'
        layer = alpha * np.dstack((mask, np.zeros_like(mask), np.zeros_like(mask)))
    return layer


def slim_seeds(seeds, sliceId=2):
    slims = np.zeros_like(seeds)
    if sliceId == 0:
        for i in range(seeds.shape[0]):
            layer = seeds[i,:,:]
            labels = skimor.label(layer, neighbors=4, background=0) + 1
            n_labels = labels.max()
            for o in range(1,n_labels+1):
                centroid = np.round(skimea.regionprops(labels == o)[0].centroid)
                slims[i, centroid[0], centroid[1]] = 1
    return slims


def crop_to_bbox(im, mask):
    if im.ndim == 2:
        # obj_rp = skimea.regionprops(mask.astype(np.integer), properties=('BoundingBox'))
        obj_rp = skimea.regionprops(mask.astype(np.integer))
        bbox = obj_rp[0].bbox # minr, minc, maxr, maxc

        bbox = np.array(bbox)
        # okrajove podminky
        bbox[0] = max(0, bbox[0]-1)
        bbox[1] = max(0, bbox[1]-1)
        bbox[2] = min(im.shape[0], bbox[2]+1)
        bbox[3] = min(im.shape[1], bbox[3]+1)

        # im = im[bbox[0]-1:bbox[2] + 1, bbox[1]-1:bbox[3] + 1]
        # mask = mask[bbox[0]-1:bbox[2] + 1, bbox[1]-1:bbox[3] + 1]
        im = im[bbox[0]:bbox[2], bbox[1]:bbox[3]]
        mask = mask[bbox[0]:bbox[2], bbox[1]:bbox[3]]
    elif im.ndim == 3:
        coords = np.nonzero(mask)
        s_min = max(0, min(coords[0]) - 1)
        s_max = min(im.shape[0], max(coords[0]) + 2)
        r_min = max(0, min(coords[1]) - 1)
        r_max = min(im.shape[1], max(coords[1]) + 2)
        c_min = max(0, min(coords[2]) - 1)
        c_max = min(im.shape[2], max(coords[2]) + 2)

        # im = im[r_min-1:r_max+1, c_min-1:c_max+1, s_min-1:s_max+1]
        # mask = mask[r_min-1:r_max+1, c_min-1:c_max+1, s_min-1:s_min+1]
        im = im[s_min:s_max, r_min:r_max, c_min:c_max]
        mask = mask[s_min:s_max, r_min:r_max, c_min:c_max]

    return im, mask


def slics_3D(im, pseudo_3D=True, n_segments=100, get_slicewise=False):
    if im.ndim != 3:
        raise Exception('3D image is needed.')

    if not pseudo_3D:
        # need to convert to RGB image
        im_rgb = np.zeros((im.shape[0], im.shape[1], im.shape[2], 3))
        im_rgb[:,:,:,0] = im
        im_rgb[:,:,:,1] = im
        im_rgb[:,:,:,2] = im

        suppxls = skiseg.slic(im_rgb, n_segments=n_segments, spacing=(2,1,1))

    else:
        suppxls = np.zeros(im.shape)
        if get_slicewise:
            suppxls_slicewise = np.zeros(im.shape)
        offset = 0
        for i in range(im.shape[0]):
            suppxl = skiseg.slic(cv2.cvtColor(im[i,:,:], cv2.COLOR_GRAY2RGB), n_segments=n_segments)
            suppxls[i,:,:] = suppxl + offset
            if get_slicewise:
                suppxls_slicewise[i,:,:] = suppxl
            offset = suppxls.max() + 1

    if get_slicewise:
        return suppxls, suppxls_slicewise
    else:
        return suppxls


def get_superpxl_intensities(im, suppxls):
    """Calculates mean intensities of pixels in superpixels
    inputs:
        im ... grayscale image, ndarray [MxN]
        suppxls ... image with suppxls labels, ndarray -same shape as im
    outputs:
        suppxl_intens ... vector with suppxls mean intensities
    """
    n_suppxl = np.int(suppxls.max() + 1)
    suppxl_intens = np.zeros(n_suppxl)

    for i in range(n_suppxl):
        sup = suppxls == i
        vals = im[np.nonzero(sup)]
        try:
            suppxl_intens[i] = np.mean(vals)
        except:
            suppxl_intens[i] = -1

    return suppxl_intens


def suppxl_ints2im(suppxls, suppxl_ints=None, im=None):
    """Replaces superpixel labels with their mean value.
    inputs:
        suppxls ... image with suppxls labels, ndarray
        suppxl_intens ... vector with suppxls mean intensities
        im ... input image
    outputs:
        suppxl_ints_im ... image with suppxls mean intensities, ndarray same shape as suppxls
    """

    suppxl_ints_im = np.zeros(suppxls.shape)

    if suppxl_ints is None and im is not None:
        suppxl_ints = get_superpxl_intensities(im, suppxls)

    for i in np.unique(suppxls):
        sup = suppxls == i
        val = suppxl_ints[i]
        suppxl_ints_im[np.nonzero(sup)] = val

    return suppxl_ints_im


def remove_empty_suppxls(suppxls):
    """Remove empty superpixels. Sometimes there are superpixels(labels), which are empty. To overcome subsequent
    problems, these empty superpixels should be removed.
    inputs:
        suppxls ... image with suppxls labels, ndarray [MxN]-same size as im
    outputs:
        new_supps ... image with suppxls labels, ndarray [MxN]-same size as im, empty superpixel labels are removed
    """
    n_suppxls = np.int(suppxls.max() + 1)
    new_supps = np.zeros(suppxls.shape, dtype=np.integer)
    idx = 0
    for i in range(n_suppxls):
        sup = suppxls == i
        if sup.any():
            new_supps[np.nonzero(sup)] = idx
            idx += 1
    return new_supps


def label_3D(data, class_labels, background=-1):
    # class_labels = np.unique(data[data > background])
    labels = - np.ones(data.shape, dtype=np.int)
    curr_l = 0
    for c in class_labels:
        x = data == c
        labs, n_labels = scindimea.label(x)
        print 'labels: ', np.unique(labs)
        # py3DSeedEditor.py3DSeedEditor(labs).show()
        for l in range(n_labels + 1):
            labels = np.where(labs == l, curr_l, labels)
            curr_l += 1
    print 'min = %i, max = %i' % (labels.min(), labels.max())
    return labels


def load_pickle_data(fname, win_level=50, win_width=350, slice_idx=-1):
    fcontent = None
    try:
        import gzip
        f = gzip.open(fname, 'rb')
        fcontent = f.read()
        f.close()
    except Exception as e:
        f = open(fname, 'rb')
        fcontent = f.read()
        f.close()
    data_dict = pickle.loads(fcontent)

    data = windowing(data_dict['data3d'], level=win_level, width=win_width)

    mask = data_dict['segmentation']

    voxel_size = data_dict['voxelsize_mm']


    if slice_idx != -1:
        data = data[slice_idx, :, :]
        mask = mask[slice_idx, :, :]

    return data, mask, voxel_size



__author__ = 'tomas'

import sys
from PyQt4 import QtGui, QtCore
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
import matplotlib.pyplot as plt
import numpy as np
import skimage.exposure as skiexp
import scipy.stats as scista

from hist_widget_GUI import Ui_Form

# Main widget containing figures etc
class Hist_widget(QtGui.QWidget):

    heal_parameter_changed = QtCore.pyqtSignal(int, int, basestring)
    hypo_parameter_changed = QtCore.pyqtSignal(int, int, basestring)
    hyper_parameter_changed = QtCore.pyqtSignal(int, int, basestring)

    def __init__(self, data=None, mask=None, models=None, hist=None, bins=None, parent=None, unaries_as_cdf=False, params=None):
        QtGui.QWidget.__init__(self, parent)
        self.ui = Ui_Form()
        self.ui.setupUi(self)
        self.setup_ui()

        self.params = params
        self.hist = hist
        self.bins = bins
        # if self.hist is None and self.bins is None:
        #     if self.data is not None:
        #         # print 'No data given - returning.'
        #         # return
        #         self.hist, self.bins = skiexp.histogram(self.data, nbins=256)

        # self.models = models  # color models
        #
        # if self.models is not None:
        #     self.rv_heal = models['heal']
        #     self.rv_hypo = models['hypo']
        #     self.rv_hyper = models['hyper']
        #     self.setup_ui()
        # else:
        #     self.rv_heal = None
        #     self.rv_hypo = None
        #     self.rv_hyper = None

        self.rv_heal = scista.norm(0, 1)
        self.rv_hypo = scista.norm(0, 1)
        self.rv_hyper = scista.norm(0, 1)

        self.unaries_as_cdf = unaries_as_cdf

        self.figure = plt.figure()
        self.axes = self.figure.add_axes([0.08, 0.05, 0.91, 0.9])
        self.canvas = FigureCanvas(self.figure)

        layout = QtGui.QVBoxLayout()
        layout.addWidget(self.canvas, 1)
        self.ui.histogram_F.setLayout(layout)

        if data is not None:
            self.set_data(data)#, mask)
        if models is not None:
            self.set_models(models)
        else:
            self.models = None

        # seting up min and max values

        # if self.params and self.params.has_key('data_min'):
        #     self.data_min = self.params['data_min']
        # elif self.data is not None:
        #     self.data_min = self.data.min()
        # else:
        #     self.data_min = 0
        # if self.params and self.params.has_key('datam_max'):
        #     self.data_max = self.params['data_max']
        # elif self.data is not None:
        #     self.data_max = self.data.max()
        # else:
        #     self.data_max = 0
        #
        # self.ui.hypo_mean_SL.setMinimum(self.data_min)
        # self.ui.heal_mean_SL.setMinimum(self.data_min)
        # self.ui.hyper_mean_SL.setMinimum(self.data_min)
        #
        # self.ui.hypo_mean_SL.setMaximum(self.data_max)
        # self.ui.heal_mean_SL.setMaximum(self.data_max)
        # self.ui.hyper_mean_SL.setMaximum(self.data_max)
        #
        # self.update_figures()

    def set_data(self, data):#, mask):
        self.data = data
        # self.mask = mask
        # if self.data is not None:
        self.hist, self.bins = skiexp.histogram(self.data, nbins=1000)
            # if mask is not None:
            #     self.data_m = self.data[np.nonzero(self.mask)]
            # else:
            #     self.data_m = self.data

        if self.params and self.params.has_key('data_min'):
            self.data_min = self.params['data_min']
        # elif self.data is not None:
        self.data_min = self.data.min()
        # else:
        #     self.data_min = 0
        if self.params and self.params.has_key('datam_max'):
            self.data_max = self.params['data_max']
        # elif self.data is not None:
        self.data_max = self.data.max()
        # else:
        #     self.data_max = 0

        self.ui.hypo_mean_SL.setMinimum(self.data_min)
        self.ui.heal_mean_SL.setMinimum(self.data_min)
        self.ui.hyper_mean_SL.setMinimum(self.data_min)

        self.ui.hypo_mean_SL.setMaximum(self.data_max)
        self.ui.heal_mean_SL.setMaximum(self.data_max)
        self.ui.hyper_mean_SL.setMaximum(self.data_max)

        self.update_figures()

    def set_models(self, models):
        self.models = models  # color models

        if self.models is not None:
            self.rv_heal = models['heal']
            self.rv_hypo = models['hypo']
            self.rv_hyper = models['hyper']
            # self.setup_ui()
            self.setup_ranges()

            self.update_heal_rv(self.rv_heal)
            self.update_hypo_rv(self.rv_hypo)
            self.update_hyper_rv(self.rv_hyper)
            self.update_figures()
        else:
            self.rv_heal = None
            self.rv_hypo = None
            self.rv_hyper = None

    def setup_ranges(self):
        # filling line edits and spinboxes
        self.ui.hypo_mean_LE.setText('%i'%int(self.rv_hypo.mean()))
        self.ui.hypo_mean_SL.setValue(int(self.rv_hypo.mean()))
        self.ui.hypo_std_SB.setValue(self.rv_hypo.std())

        self.ui.heal_mean_LE.setText('%i'%int(self.rv_heal.mean()))
        self.ui.heal_mean_SL.setValue(int(self.rv_heal.mean()))
        self.ui.heal_std_SB.setValue(self.rv_heal.std())

        self.ui.hyper_mean_LE.setText('%i'%int(self.rv_hyper.mean()))
        self.ui.hyper_mean_SL.setValue(int(self.rv_hyper.mean()))
        self.ui.hyper_std_SB.setValue(self.rv_hyper.std())

    def setup_ui(self):
        # self.setup_ranges()

        # callbacks - hypodense mean
        self.ui.hypo_mean_SL.valueChanged.connect(self.hypo_mean_SL_callback)
        hypo_mean_val = QtGui.QIntValidator(self.ui.hypo_mean_SL.minimum(), self.ui.hypo_mean_SL.maximum())
        self.ui.hypo_mean_LE.setValidator(hypo_mean_val)
        self.ui.hypo_mean_LE.textChanged.connect(self.hypo_mean_LE_callback)

        # callbacks - heal mean
        self.ui.heal_mean_SL.valueChanged.connect(self.heal_mean_SL_callback)
        heal_mean_val = QtGui.QIntValidator(self.ui.heal_mean_SL.minimum(), self.ui.heal_mean_SL.maximum())
        self.ui.heal_mean_LE.setValidator(heal_mean_val)
        self.ui.heal_mean_LE.textChanged.connect(self.heal_mean_LE_callback)

        # callbacks - hyper mean
        self.ui.hyper_mean_SL.valueChanged.connect(self.hyper_mean_SL_callback)
        hyper_mean_val = QtGui.QIntValidator(self.ui.hyper_mean_SL.minimum(), self.ui.hyper_mean_SL.maximum())
        self.ui.hyper_mean_LE.setValidator(hyper_mean_val)
        self.ui.hyper_mean_LE.textChanged.connect(self.hyper_mean_LE_callback)

        # callbacks - stds
        self.ui.hypo_std_SB.valueChanged.connect(self.hypo_std_SB_callback)
        self.ui.hyper_std_SB.valueChanged.connect(self.hyper_std_SB_callback)
        self.ui.heal_std_SB.valueChanged.connect(self.heal_std_SB_callback)

    # def set_models(self, models):
    #     self.models = models
    #
    #     self.rv_heal = models['heal']
    #     self.rv_hypo = models['hypo']
    #     self.rv_hyper = models['hyper']
    #
    #     self.setup_ui()
    #
    #     self.update_figures()

    def hypo_mean_SL_callback(self, value):
        self.rv_hypo = scista.norm(value, self.rv_hypo.std())
        self.update_hypo_rv(self.rv_hypo)
        self.update_figures()

    def heal_mean_SL_callback(self, value):
        self.rv_heal = scista.norm(value, self.rv_heal.std())
        self.update_heal_rv(self.rv_heal)
        self.update_figures()

    def hyper_mean_SL_callback(self, value):
        self.rv_hyper = scista.norm(value, self.rv_hyper.std())
        self.update_hyper_rv(self.rv_hyper)
        self.update_figures()

    def hypo_mean_LE_callback(self, value):
        try:  # must be due to the possibility that no character could be entered
            self.ui.hypo_mean_SL.setValue(int(value))
        except:
            pass

    def heal_mean_LE_callback(self, value):
        try:  # must be due to the possibility that no character could be entered
            self.ui.heal_mean_SL.setValue(int(value))
        except:
            pass

    def hyper_mean_LE_callback(self, value):
        try:  # must be due to the possibility that no character could be entered
            self.ui.hyper_mean_SL.setValue(int(value))
        except:
            pass

    def hypo_std_SB_callback(self, value):
        # self.statusBar().showMessage('Hypodense model updated thru spin box.')
        self.rv_hypo = scista.norm(self.rv_hypo.mean(), value)
        self.update_hypo_rv(self.rv_hypo)
        self.update_figures()

    def hyper_std_SB_callback(self, value):
        # self.statusBar().showMessage('Hyperdense model updated thru spin box.')
        self.rv_hyper = scista.norm(self.rv_hyper.mean(), value)
        self.update_hyper_rv(self.rv_hyper)
        self.update_figures()

    def heal_std_SB_callback(self, value):
        # self.statusBar().showMessage('Healthy model updated thru spin box.')
        self.rv_heal = scista.norm(self.rv_heal.mean(), value)
        self.update_heal_rv(self.rv_heal)
        self.update_figures()

    def update_heal_rv(self, new_rv):
        self.rv_heal = new_rv
        if int(self.ui.heal_mean_LE.text()) != self.rv_heal.mean():
            self.ui.heal_mean_LE.setText('%i'%int(self.rv_heal.mean()))
        if self.ui.heal_std_SB.value() != self.rv_heal.std():
            self.ui.heal_std_SB.setValue(self.rv_heal.std())
        self.heal_parameter_changed.emit(new_rv.mean(), new_rv.std(), 'heal')

    def update_hypo_rv(self, new_rv):
        self.rv_hypo = new_rv
        if int(self.ui.hypo_mean_LE.text()) != self.rv_hypo.mean():
            self.ui.hypo_mean_LE.setText('%i'%int(self.rv_hypo.mean()))
        if self.ui.hypo_std_SB.value() != self.rv_hypo.std():
            self.ui.hypo_std_SB.setValue(self.rv_hypo.std())
        self.hypo_parameter_changed.emit(new_rv.mean(), new_rv.std(), 'hypo')

    def update_hyper_rv(self, new_rv):
        self.rv_hyper = new_rv
        if int(self.ui.hyper_mean_LE.text()) != self.rv_hyper.mean():
            self.ui.hyper_mean_LE.setText('%i'%int(self.rv_hyper.mean()))
        if self.ui.hyper_std_SB.value() != self.rv_hyper.std():
            self.ui.hyper_std_SB.setValue(self.rv_hyper.std())
        self.hyper_parameter_changed.emit(new_rv.mean(), new_rv.std(), 'hyper')

    def update_figures(self):
        plt.figure(self.figure.number)
        x = np.arange(self.data.min(), self.data.max())#, (self.data.max() - self.data.min()) / 100)  # artificial x-axis
        # self.figure.gca().cla()  # clearing the figure, just to be sure

        # plt.subplot(411)
        plt.plot(self.bins, self.hist, 'k')
        plt.hold(True)
        # if self.rv_heal is not None and self.rv_hypo is not None and self.rv_hyper is not None:
        if self.models is not None:
            healthy_y = self.rv_heal.pdf(x)
            if self.unaries_as_cdf:
                hypo_y = (1 - self.rv_hypo.cdf(x)) * self.rv_heal.pdf(self.rv_heal.mean())
                hyper_y = self.rv_hyper.cdf(x) * self.rv_heal.pdf(self.rv_heal.mean())
            else:
                hypo_y = self.rv_hypo.pdf(x)
                hyper_y = self.rv_hyper.pdf(x)
            y_max = max(healthy_y.max(), hypo_y.max(), hyper_y.max())
            fac = self.hist.max() / y_max

            plt.plot(x, fac * healthy_y, 'g', linewidth=2)
            plt.plot(x, fac * hypo_y, 'b', linewidth=2)
            plt.plot(x, fac * hyper_y, 'r', linewidth=2)
        ax = plt.axis()
        # plt.axis([0, 256, ax[2], ax[3]])
        plt.gca().tick_params(direction='in', pad=1)
        plt.hold(False)
        # plt.grid(True)

        self.canvas.draw()

    def keyPressEvent(self, QKeyEvent):
        print 'hist widget key event: ',
        if QKeyEvent.key() == QtCore.Qt.Key_Escape:
            print 'Escape'
            self.close()

if __name__ == '__main__':

    from hist_widget_GUI import Ui_Form
    import skimage.io as skiio
    import skimage.exposure as skiexp
    import scipy.stats as scista

    app = QtGui.QApplication(sys.argv)

    img = skiio.imread('/home/tomas/Dropbox/images/puma.png', as_grey=True)
    img = skiexp.rescale_intensity(img, in_range=(0, 1), out_range=np.uint8)

    models = dict()
    rv_hypo = scista.norm(50, 4)
    rv_heal = scista.norm(125, 10)
    rv_hyper = scista.norm(200, 5)
    models['hypo'] = rv_hypo
    models['heal'] = rv_heal
    models['hyper'] = rv_hyper
    params = {'data_min': img.min(), 'data_max':img.max()}
    hist_w = Hist_widget(data=img, params=params, unaries_as_cdf=True)
    hist_w.set_models(models)
    hist_w.show()

    sys.exit(app.exec_())


__author__ = 'tomas'

import numpy as np
import tools

PRIORITY_LOW = 0  # for lessions that are extracted autonomously
PRIORITY_HIGH = 1  # for lessions that are added by the user, these wil not be filtrated by sliders (area, density, ...)

def create_lesion_from_pt(pt, density, lbl, priority=PRIORITY_HIGH):
    """

    :param center: center of lesion, [s, x, y] = [s, c, r]
    :param density:
    :param lbl:
    :return:
    """
    les = Lesion(lbl)

    les.area = 1
    les.compactness = 1
    les.center = pt
    les.priority = priority
    les.mean_density = density
    les.std_density = 0

    les.max_width = 1
    les.max_height = 1
    les.max_depth = 1
    # minimal and maximal row, column and slice
    les.r_min = pt[1]
    les.r_max = pt[1]
    les.c_min = pt[2]
    les.c_max = pt[2]
    les.s_min = pt[0]
    les.s_max = pt[0]

    les.hist = None  # histogram of density

    les.chord = 1  # longest chord (tetiva in czech)

    return les

class Lesion(object):
    """ This class represents lesions. """

    def __init__(self, label, mask=None, data=None, priority=PRIORITY_LOW):
        self.label = label  # label of the lesion in segmented data; its identifier

        self.area = None  # area of the lesion

        self.compactness = None

        self.center = None  # center of mass

        self.priority = priority

        self.mean_density = None
        self.std_density = None

        self.max_width = None
        self.max_height = None
        self.max_depth = None

        # minimal and maximal row, column and slice
        self.r_min = None
        self.r_max = None
        self.c_min = None
        self.c_max = None
        self.s_min = None
        self.s_max = None

        self.hist = None  # histogram of density

        self.chord = None  # longest chord (tetiva in czech)

        if mask is not None:
            self.compute_features(mask, data)

    def compute_features(self, mask, data):
        # getting unique labels that are greater than 0 (0 = background, -1 = out of mask)
        self.area = mask.sum()

        s, r, c = np.nonzero(mask)

        self.compactness = tools.get_zunics_compatness(mask)

        self.center = (s.mean(), r.mean(), c.mean())

        if data is not None:
            pts = data[np.nonzero(mask)]
            self.mean_density = pts.mean()
            self.mean_density_std = pts.std()

        self.r_min = r.min()
        self.r_max = r.max()
        self.c_min = c.min()
        self.c_max = c.max()
        self.s_min = s.min()
        self.s_max = s.max()

        self.max_width = self.c_max - self.c_min
        self.max_height = self.r_max - self.r_min
        self.max_depth = self.c_max - self.c_min

    def __str__(self):
        return 'label=%i, area=%i, mean_dens=%.2f, mean_dens_std=%.2f, center=[%.1f, %.1f, %.1f]' % (
            self.label, self.area, self.mean_density, self.mean_density_std, self.center[0], self.center[1], self.center[2])

def extract_lesions(labels, data=None):
    """
    For each label in 'labels' it creates an instance. Returns list of lesions.
    :param labels: labeled data, lesions have label > 0 (0 = background, -1 = points outside a mask)
    :return: list of lesions
    """
    lesions = list()
    lb_list = [x for x in np.unique(labels) if x > 0]

    for i in lb_list:
        im = labels == i
        lesion = Lesion(i, mask=im, data=data)
        lesions.append(lesion)

    return lesions


if __name__ == '__main__':
    labels = np.array([[1, 1, 0, 2, 0],
                       [1, 1, 0, 2, 0],
                       [0, 0, 0, 2, 0],
                       [3, 0, 4, 0, 5],
                       [3, 0, 4, 0, 0]], dtype=np.int)
    labels = np.dstack((labels, labels, labels))

    lesions = extract_lesions(labels, data=labels)

    for i in lesions:
        print i

# TODO: dopocitat compactness
# TODO: dopocitat chord
# TODO: dopocitat hist


from PyQt4.QtCore import *
from PyQt4.QtGui import *
import sys
import numpy as np


class MyTableModel(QAbstractTableModel):
    def __init__(self, datain, headerdata=None, parent=None, *args):
        QAbstractTableModel.__init__(self, parent, *args)

        if headerdata is None:
            headerdata = ['label', 'area', 'compactness']

        self.data = datain
        self.headerdata = headerdata

    def rowCount(self, parent):
        # Number of rows corresponds to the number of objects
        return self.data.shape[0]

    def columnCount(self, parent):
        # Number of columns corresponds to the number of features
        return self.data.shape[1]

    def data(self, index, role):
        if not index.isValid():
            return QVariant()
        elif role != Qt.DisplayRole:
            return QVariant()
        return QVariant('%.3f'%self.data[index.row(), index.column()])

    def headerData(self, idx, orientation, role):
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            return QVariant(self.headerdata[idx])
        if orientation == Qt.Vertical and role == Qt.DisplayRole:
            return QVariant(idx)
        return QVariant()



class MyWindow(QWidget):
    def __init__(self, data, header=None, *args):
        QWidget.__init__(self, *args)

        self.data = data
        self.tablemodel = MyTableModel(self.data, header, self)
        self.tableview = QTableView()
        self.tableview.setModel(self.tablemodel)
        self.tableview.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.tableview.selectionModel().selectionChanged.connect(self.selection_changed)

        add_BTN = QPushButton('add')
        self.connect(add_BTN, SIGNAL('clicked()'), self.add_row)

        del_BTN = QPushButton('delete')
        self.connect(del_BTN, SIGNAL('clicked()'), self.del_row)

        layoutV = QVBoxLayout(self)
        layoutV.addWidget(self.tableview)

        layoutH = QHBoxLayout(self)
        layoutH.addWidget(add_BTN)
        layoutH.addWidget(del_BTN)

        btn_frame = QFrame()
        btn_frame.setLayout(layoutH)

        layoutV.addWidget(btn_frame)
        self.setLayout(layoutV)

    def selection_changed(self, selected, deselected):
        indexes = self.tableview.selectionModel().selectedRows()
        for index in indexes:
            print index.row()
            # self.tableview.selectRow(index.row())

    def add_row(self):
        try:
            new_idx = self.data[:, 0].max() + 1
            self.data = np.vstack((self.data, np.array([new_idx, 0, 0])))
        except ValueError:
            self.data = np.array([[1, 0, 0]])
        self.tablemodel.data = self.data
        self.tableview.model().layoutChanged.emit()
        print 'row added'

    def del_row(self):
        self.data = self.data[:-1, :]
        self.tablemodel.data = self.data
        self.tableview.model().layoutChanged.emit()
        print 'row deleted'


if __name__ == "__main__":

    # my_array = [['1', '00', '01'],
    #             ['2', '10', '11'],
    #             ['3', '20', '21']]

    areas = [10, 20, 30, 40]
    comps = [0.1, 0.2, 0.3, 0.4]
    idxs = np.arange(1, len(areas)+1)
    data = np.vstack((idxs, areas, comps)).T

    app = QApplication(sys.argv)
    w = MyWindow(data)
    w.show()
    sys.exit(app.exec_())


__author__ = 'tomas'

import numpy as np
import matplotlib.pyplot as plt

import skimage.io as skiio
import skimage.transform as skitra
import skimage.exposure as skiexp
import skimage.segmentation as skiseg
from sklearn import mixture

import scipy.stats as scista

from pygco import cut_from_graph

import itertools


fname = '/home/tomas/Dropbox/images/Berkeley_Benchmark/man_with_hat_189080.jpg'
im = skiio.imread(fname, as_grey=True)
sf = 0.5
im = skiexp.rescale_intensity(skitra.rescale(im, sf), (0, 1), (0, 255)).astype(np.int)
pixs = im.flatten()

# estimating number of components
lowest_bic = np.infty
bic = []
max_n_components = 5
n_components_range = range(1, max_n_components + 1)
cv_types = ['spherical']#, 'tied', 'diag', 'full']
for cv_type in cv_types:
    print 'cv_type: ', cv_type
    for n_components in n_components_range:
        print '\tn_components: ', n_components
        # Fit a mixture of Gaussians with EM
        gmm = mixture.GMM(n_components=n_components, covariance_type=cv_type)
        gmm.fit(pixs)
        bic.append(gmm.bic(pixs))
        if bic[-1] < lowest_bic:
            lowest_bic = bic[-1]
            best_gmm = gmm
n_components = n_components_range[np.argmin(bic)]

print 'bic score:', bic
print 'optimal number of components: %i, bic: %.1f' % (n_components, lowest_bic)
print 'means:\n', best_gmm.means_
print 'covars:\n', best_gmm.covars_

labels = best_gmm.predict(pixs)
labels = labels.reshape(im.shape)

# labeling
probs = np.zeros((im.shape[0], im.shape[1], n_components))
for i in range(n_components):
    rv = scista.norm(best_gmm.means_[i], np.sqrt(best_gmm.covars_[i]))
    probs[:, :, i] = rv.pdf(pixs).reshape(im.shape)

    # plt.figure()
    # plt.imshow(probs[:, :, i], 'gray')
    # plt.title('component #%i' % i)
# plt.show()


# MRF
#   unary term
beta = 1
unaries = np.zeros((im.shape[0], im.shape[1], n_components))
for i in range(n_components):
    rv = scista.norm(best_gmm.means_[i], np.sqrt(best_gmm.covars_[i]))
    unaries[:, :, i] = - beta * rv.logpdf(im)
unaries = unaries.astype(np.int32)

for i in range(n_components):
    plt.figure()
    plt.subplot(121), plt.imshow(probs[:, :, i], 'gray'), plt.title('probability for component #%i' % i)
    plt.subplot(122), plt.imshow(unaries[:, :, i], 'gray'), plt.title('unary term for component #%i' % i)
# plt.show()

#   pairwise term
alpha = 50
pairwise = - alpha * np.eye(n_components, dtype=np.int32)

print 'deriving graph edges...'
# use the general graph algorithm
# first, we construct the grid graph
inds = np.arange(im.size).reshape(im.shape)
horz = np.c_[inds[:, :-1].ravel(), inds[:, 1:].ravel()]
vert = np.c_[inds[:-1, :].ravel(), inds[1:, :].ravel()]
edges = np.vstack([horz, vert]).astype(np.int32)

# we flatten the unaries
result_graph = cut_from_graph(edges, unaries.reshape(-1, n_components), pairwise)
res = result_graph.reshape(im.shape)

plt.figure()
plt.subplot(121), plt.imshow(im, 'gray'), plt.title('input image')
plt.subplot(122), plt.imshow(res, 'gray'), plt.title('segmentation')
# plt.show()

# plotting results
color_iter = itertools.cycle(['r', 'g', 'b', 'c', 'm', 'y', 'k'])
x = np.arange(0, 255, 0.1)
hist, bins = skiexp.histogram(pixs, nbins=256)

plt.figure()
plt.subplot(211), plt.plot(bins, hist)
ax = plt.axis()
plt.axis([0, 256, ax[2], ax[3]])
plt.subplot(212)
for i in range(n_components):
    rv = scista.norm(best_gmm.means_[i], np.sqrt(best_gmm.covars_[i]))
    plt.plot(x, rv.pdf(x), color_iter.next())
ax = plt.axis()
plt.axis([0, 256, ax[2], ax[3]])


plt.figure()
plt.subplot(121), plt.imshow(im, 'gray')
plt.subplot(122), plt.imshow(labels, interpolation='nearest')

plt.show()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'lession_editor_GUI_slim2.ui'
#
# Created: Tue Sep  8 10:43:56 2015
#      by: PyQt4 UI code generator 4.10.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName(_fromUtf8("MainWindow"))
        MainWindow.resize(720, 839)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        MainWindow.setTabShape(QtGui.QTabWidget.Rounded)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.centralwidget.setObjectName(_fromUtf8("centralwidget"))
        self.verticalLayout_4 = QtGui.QVBoxLayout(self.centralwidget)
        self.verticalLayout_4.setObjectName(_fromUtf8("verticalLayout_4"))
        self.viewer_F = QtGui.QFrame(self.centralwidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.viewer_F.sizePolicy().hasHeightForWidth())
        self.viewer_F.setSizePolicy(sizePolicy)
        self.viewer_F.setMinimumSize(QtCore.QSize(0, 0))
        self.viewer_F.setFrameShape(QtGui.QFrame.Box)
        self.viewer_F.setObjectName(_fromUtf8("viewer_F"))
        self.verticalLayout_4.addWidget(self.viewer_F)
        MainWindow.setCentralWidget(self.centralwidget)
        self.action_Load_serie_1 = QtGui.QAction(MainWindow)
        self.action_Load_serie_1.setObjectName(_fromUtf8("action_Load_serie_1"))
        self.action_Load_serie_2 = QtGui.QAction(MainWindow)
        self.action_Load_serie_2.setObjectName(_fromUtf8("action_Load_serie_2"))
        self.actionRun = QtGui.QAction(MainWindow)
        self.actionRun.setObjectName(_fromUtf8("actionRun"))
        self.actionDelete = QtGui.QAction(MainWindow)
        self.actionDelete.setObjectName(_fromUtf8("actionDelete"))
        self.actionRestart = QtGui.QAction(MainWindow)
        self.actionRestart.setObjectName(_fromUtf8("actionRestart"))
        self.actionColor_Model = QtGui.QAction(MainWindow)
        self.actionColor_Model.setObjectName(_fromUtf8("actionColor_Model"))
        self.actionObject_List = QtGui.QAction(MainWindow)
        self.actionObject_List.setObjectName(_fromUtf8("actionObject_List"))
        self.actionCircle = QtGui.QAction(MainWindow)
        self.actionCircle.setObjectName(_fromUtf8("actionCircle"))
        self.actionRuler = QtGui.QAction(MainWindow)
        self.actionRuler.setObjectName(_fromUtf8("actionRuler"))
        self.actionAxial = QtGui.QAction(MainWindow)
        self.actionAxial.setObjectName(_fromUtf8("actionAxial"))
        self.actionFrontal = QtGui.QAction(MainWindow)
        self.actionFrontal.setObjectName(_fromUtf8("actionFrontal"))
        self.actionSagital = QtGui.QAction(MainWindow)
        self.actionSagital.setObjectName(_fromUtf8("actionSagital"))

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow", None))
        self.action_Load_serie_1.setText(_translate("MainWindow", "Load serie #1...", None))
        self.action_Load_serie_2.setText(_translate("MainWindow", "Load serie #2...", None))
        self.actionRun.setText(_translate("MainWindow", "Run", None))
        self.actionDelete.setText(_translate("MainWindow", "Delete", None))
        self.actionRestart.setText(_translate("MainWindow", "Restart", None))
        self.actionColor_Model.setText(_translate("MainWindow", "Color Model", None))
        self.actionObject_List.setText(_translate("MainWindow", "Object List", None))
        self.actionCircle.setText(_translate("MainWindow", "Circle", None))
        self.actionRuler.setText(_translate("MainWindow", "Ruler", None))
        self.actionAxial.setText(_translate("MainWindow", "Axial", None))
        self.actionFrontal.setText(_translate("MainWindow", "Frontal", None))
        self.actionSagital.setText(_translate("MainWindow", "Sagital", None))




__author__ = 'tomas'

import numpy as np
import tools
import matplotlib.pyplot as plt
from skimage import graph, data, io, color
import skimage.segmentation as skiseg

slice_idx = 60
# fname = '/home/tomas/Data/liver_segmentation_06mm/tryba/data_other/org-exp_186_49290986_arterial_0.6_B20f-.pklz'

# im, mask, voxel_size = tools.load_pickle_data(fname, slice_idx=slice_idx)

fname = '/home/tomas/Dropbox/images/medicine/hypo2in.png'
im = io.imread(fname, as_grey=True)

labels = skiseg.slic(np.dstack((im, im, im)), compactness=10, n_segments=100)
g = graph.rag_mean_color(im, labels, mode='similarity')
nc = graph.cut_normalized(labels, g)
out2 = color.label2rgb(nc, im, kind='avg')

plt.figure()
plt.subplot(121), plt.imshow(nc)
plt.subplot(122), plt.imshow(out2)
# io.show()

plt.figure()
plt.imshow(im, 'gray')
plt.show()


__author__ = 'tomas'

import numpy as np
import matplotlib.pyplot as plt

import sys

from PyQt4.QtCore import Qt, QSize, QString, SIGNAL, QPoint, pyqtSignal
from PyQt4.QtGui import QImage, QDialog,\
    QApplication, QSlider, QPushButton,\
    QLabel, QPixmap, QPainter, qRgba,\
    QComboBox, QIcon, QStatusBar,\
    QHBoxLayout, QVBoxLayout, QFrame, QSizePolicy,\
    QPen, QMainWindow, QWidget

import skimage.morphology as skimor
import skimage.io as skiio

import area_hist_widget as ahw


# BGRA order
GRAY_COLORTABLE = np.array([[ii, ii, ii, 255] for ii in range(256)],
                           dtype=np.uint8)

SEEDS_COLORTABLE = np.array([[0, 255, 0, 255],
                             [0, 0, 255, 255]], dtype=np.uint8)

LABELS_COLORTABLE = np.array([[0, 0, 0, 255],
                              [255, 0, 0, 255],
                              [0, 255, 0, 255],
                              [0, 0, 255, 255]], dtype=np.uint8)

CONTOURS_COLORS = {
    1: [255, 0, 0],
    2: [0, 255, 0],
    3: [0, 0, 255],
    # 10: [255, 255, 0],
    # 11: [0, 255, 0],
    # 12: [0, 255, 255],
    # 13: [0, 0, 255],
}

# VIEW_TABLE = {'axial': (1,0),
#               'sagittal': (1,0,2),
#               'coronal': (2,0,1)}

CONTOURS_COLORTABLE = np.zeros((256,4), dtype=np.uint8)
CONTOURS_COLORTABLE[:,:3] = 255
CONTOURLINES_COLORTABLE = np.zeros((256,2,4), dtype=np.uint8)
CONTOURLINES_COLORTABLE[:,:,:3] = 255

for ii, jj in CONTOURS_COLORS.iteritems():
    key = ii - 1
    CONTOURS_COLORTABLE[key,:3] = jj
    CONTOURS_COLORTABLE[key,3] = 64
    CONTOURLINES_COLORTABLE[key,0,:3] = jj
    CONTOURLINES_COLORTABLE[key,0,3] = 0  # alpha channel of color inside a contour, 0 = no color
    CONTOURLINES_COLORTABLE[key,1,:3] = jj
    CONTOURLINES_COLORTABLE[key,1,3] = 255


NEI_TAB = [[-1, -1], [0, -1], [1, -1],
           [-1, 0], [1, 0],
           [-1, 1], [0, 1], [1, 1]]


def erase_reg(arr, p, val=0):
    from scipy.ndimage.measurements import label

    labs, num = label(arr)
    aval = labs[p]
    idxs = np.where(labs == aval)
    arr[idxs] = val


class SliceBox(QLabel):

    mouseClickSignal = pyqtSignal(list, int)

    def __init__(self, main):#, sliceSize, voxel_size, main, mode='seeds'):
    # def __init__(self, sliceSize, grid, mode='seeds'):
    # def __init__(self, sliceSize, mode='seeds'):

        QLabel.__init__(self)
        self.main = main

    def setup_widget(self, sliceSize, voxel_size):
        height = 600
        vscale = voxel_size / float(np.min(voxel_size))
        grid = height / float(sliceSize[1] * vscale[1])
        grid = (grid * vscale[0], grid * vscale[1])

        # self.drawing = False
        # self.modified = False
        # self.last_position = None
        self.imagesize = QSize(int(sliceSize[0] * grid[0]),
                               int(sliceSize[1] * grid[1]))

        # self.setMouseTracking(True)
        self.mouse_cursor = [0, 0]
        self.mouse_glob = [0, 0]
        self.circle_r = 1
        self.circle_strel = np.nonzero(skimor.disk(self.circle_r))
        self.circle_m = self.circle_strel

        self.SHOW_IM = 0  # flag for showing density data
        self.SHOW_LABELS = 1  # flag for showing labels
        self.SHOW_CONTOURS = 2  # flag for showing contours

        # self.imagesize = QSize(sliceSize[0], sliceSize[1])
        self.grid = grid
        self.grid_res = np.copy(grid)
        self.slice_size = sliceSize
        self.ctslice_rgba = None
        self.cw = {'c': 1.0, 'w': 1.0}

        self.seeds = None
        self.contours = None
        self.contours_old = None
        self.mask_points = None
        # self.contour_mode = 'fill'
        # self.contour_mode = 'contours'
        self.contours_mode_is_fill = True

        self.show_mode = self.SHOW_IM

        # self.actual_slice = 0
        # self.n_slices = 0
        self.scroll_fun = None

        self.image = QImage(self.imagesize, QImage.Format_RGB32)
        self.setPixmap(QPixmap.fromImage(self.image))
        self.setScaledContents(True)

        self.actual_view = 'axial'
        # self.act_transposition = VIEW_TABLE[self.actual_view]

        self.circle_active = False
        self.ruler_active = False

        self.area_hist_widget = None

        self.setScaledContents(True)

        # self.area_hist_widget = ahw.AreaHistWidget()
        # self.area_hist_widget.setEnabled(False)
        # self.area_hist_widget.show()

    def reinit(self, sliceSize):
        self.imagesize = QSize(int(sliceSize[0] * self.grid[0]),
                               int(sliceSize[1] * self.grid[1]))
        self.slice_size = sliceSize
        self.ctslice_rgba = None

        self.seeds = None
        self.contours = None
        self.contours_old = None
        self.mask_points = None

        self.image = QImage(self.imagesize, QImage.Format_RGB32)
        self.setPixmap(QPixmap.fromImage(self.image))
        self.setScaledContents(True)

    # def set_data(self, data, type):
    #     self.data = data
    #     self.n_slices, self.n_rows, self.n_cols = data.shape
    #     self.type = type
    #
    #     self.imagesize = QSize(int(self.n_rows * self.grid[0]),
    #                            int(self.n_cols * self.grid[1]))
    #     self.slice_size = (self.n_rows, self.n_cols)

    # def set_slice_size(self, sliceSize):
    #     self.imagesize = QSize(sliceSize[0], sliceSize[1])
    #     self.slice_size = sliceSize
    #
    #     self.seeds = None
    #     self.contours = None
    #     self.contours_old = None
    #     self.mask_points = None
    #
    #     self.image = QImage(self.imagesize, QImage.Format_RGB32)
    #     self.setPixmap(QPixmap.fromImage(self.image))
        self.setScaledContents(True)

    def setContours(self, contours):
        self.contours = contours
        self.contours_aview = self.contours.transpose(self.act_transposition)

    def paintEvent(self, event):
        painter = QPainter(self)
        painter.drawImage(event.rect(), self.image)
        painter.end()

    def get_contours(self, img, sl):
        idxs = sl.nonzero()[0]
        keys = np.unique(sl[idxs])
        for ii in keys:
            if ii == 0:
                continue
            aux = np.zeros_like(sl)
            idxsi = np.where(sl == ii)[0]
            aux[idxsi] = 1
            cnt = self.gen_contours(aux)

            self.composeRgba(img, cnt,
                             CONTOURLINES_COLORTABLE[ii - 1,...])

    def gen_contours(self, sl):
        sls = sl.reshape(self.slice_size, order='F')
        cnt = sls.copy()
        chunk = np.zeros((cnt.shape[1] + 2,), dtype=np.int8)
        for irow, row in enumerate(sls):
            chunk[1:-1] = row
            chdiff = np.diff(chunk)
            idx1 = np.where(chdiff > 0)[0]
            if idx1.shape[0] > 0:
                idx2 = np.where(chdiff < 0)[0]
                if idx2.shape[0] > 0:
                    cnt[irow,idx1] = 2
                    cnt[irow,idx2 - 1] = 2

        chunk = np.zeros((cnt.shape[0] + 2,), dtype=np.int8)
        for icol, col in enumerate(sls.T):
            chunk[1:-1] = col
            chdiff = np.diff(chunk)
            idx1 = np.where(chdiff > 0)[0]
            if idx1.shape[0] > 0:
                idx2 = np.where(chdiff < 0)[0]
                if idx2.shape[0] > 0:
                    cnt[idx1,icol] = 2
                    cnt[idx2 - 1,icol] = 2

        return cnt.ravel(order='F')

    def composeRgba(self, bg, fg, cmap):
        idxs = fg.nonzero()[0]

        if idxs.shape[0] > 0:
            fg_rgb = cmap[fg[idxs] - 1]

            af = fg_rgb[...,3].astype(np.uint32)
            rgbf = fg_rgb[...,:3].astype(np.uint32)
            rgbb = bg[idxs,:3].astype(np.uint32)

            rgbx = ((rgbf.T * af).T + (rgbb.T * (255 - af)).T) / 255
            bg[idxs,:3] = rgbx.astype(np.uint8)

    def overRgba(self, bg, fg, cmap):
        idxs = fg.nonzero()[0]
        bg[idxs] = cmap[fg[idxs] - 1]

    def window_slice(self, ctslice):
        if self.win_w > 0:
            mul = 255. / float(self.win_w)
        else:
            mul = 0

        lb =self.win_l - self.win_w / 2
        aux = (ctslice - lb) * mul
        aux = np.where(aux < 0, 0, aux)
        aux = np.where(aux > 255, 255, aux)

        return aux.astype(np.uint8)

    def updateSlice(self):
        if self.ctslice_rgba is None:
            return

        img = self.ctslice_rgba.copy()

        # if self.seeds is not None:
        #     if self.mode_draw:
        #         if self.contour_mode == 'fill':
        #             self.composeRgba(img, self.seeds,
        #                              self.seeds_colortable)
        #         elif self.contour_mode == 'contours':
        #             self.get_contours(img, self.seeds)
        #     else:
        #         self.overRgba(img, self.seeds,
        #                       self.seeds_colortable)

        if self.contours is not None:
            # if self.contour_mode == 'fill':
            if self.contours_mode_is_fill:
                self.composeRgba(img, self.contours, CONTOURS_COLORTABLE)
            # elif self.contour_mode == 'contours':
            else:
                self.get_contours(img, self.contours)

        # masking out pixels under circle
        # for i in self.circle_m:
        #     img[i, :] = [0, 0, 255, 255]

        image = QImage(img.flatten(),
                     self.slice_size[0], self.slice_size[1],
                     QImage.Format_ARGB32).scaled(self.imagesize)
        painter = QPainter(self.image)
        painter.drawImage(0, 0, image)

        if self.show_mode == self.SHOW_CONTOURS:# and self.centers is not None:
            if self.centers is not None:
                pts = self.centers.nonzero()
                pen = QPen(Qt.red, 3)
                painter.setPen(pen)
                for i in range(len(pts[0])):
                    painter.drawPoint(pts[1][i] * self.grid[0], pts[0][i] * self.grid[1])

        if self.circle_active:
            pen = QPen(Qt.red, 3)
            painter.setPen(pen)
            center_offset = 0 #0.5
            radius_offset = 0 #0.5
            painter.drawEllipse(QPoint((self.mouse_cursor[0] + center_offset) * self.grid[0], (self.mouse_cursor[1] + center_offset) * self.grid[1]),
                                (self.circle_r + radius_offset) * self.grid[0], (self.circle_r + radius_offset) * self.grid[1])
        painter.end()

        self.update()

    def getSliceRGBA(self, ctslice):
        if self.cw['w'] > 0:
            mul = 255.0 / float(self.cw['w'])

        else:
            mul = 0

        lb = self.cw['c'] - self.cw['w'] / 2
        aux = (ctslice.ravel(order='F') - lb) * mul
        idxs = np.where(aux < 0)[0]
        aux[idxs] = 0
        idxs = np.where(aux > 255)[0]
        aux[idxs] = 255

        return aux.astype(np.uint8)

    def updateSliceCW(self, ctslice=None):
        if ctslice is not None:
            self.ctslice_rgba = GRAY_COLORTABLE[self.getSliceRGBA(ctslice)]

        self.updateSlice()

    def setSlice(self, ctslice=None, seeds=None, contours=None, centers=None):
        # ctslice = 200 * np.triu(np.ones(ctslice.shape, dtype=int))
        self.ctslice = ctslice
        self.centers = centers
        # ctslice = np.transpose(ctslice)
        if ctslice is not None:
            if self.show_mode in (self.SHOW_IM, self.SHOW_CONTOURS):
                # tmp = self.getSliceRGBA(ctslice)
                self.ctslice_rgba = GRAY_COLORTABLE[self.getSliceRGBA(ctslice)]
            elif self.show_mode == self.SHOW_LABELS:
                self.ctslice_rgba = LABELS_COLORTABLE[ctslice.ravel(order='F')]

        if seeds is not None:
            self.seeds = seeds.ravel(order='F')
        else:
            self.seeds = None

        if contours is not None and self.show_mode == self.SHOW_CONTOURS:
            self.contours = contours.transpose().ravel(order='F')#self.act_transposition).ravel(order='F')
            # self.contours = contours.transpose(self.act_transposition)
        else:
            self.contours = None

        self.updateSlice()

    def gridPosition(self, pos):
        # return (int(pos.x() / self.grid[0]),
        #         int(pos.y() / self.grid[1]))
        return (int(pos.x() / self.grid_res[0]),
                int(pos.y() / self.grid_res[1]))
        # return (int(pos.x()), int(pos.y()))

    def resizeSlice(self, new_slice_size=None, new_grid=None, new_image_size=None):

        if new_slice_size is not None:
            self.slice_size = new_slice_size

        if new_grid is not None:
            self.grid = new_grid

        if new_image_size is not None:
            self.imagesize = new_image_size
        else:
            self.imagesize = QSize(int(self.slice_size[0] * self.grid[0]),
                                   int(self.slice_size[1] * self.grid[1]))
            # self.imagesize = QSize(self.slice_size[0], self.slice_size[1])
        # self.image = QImage(self.imagesize, QImage.Format_RGB32)
        # pixmap = QPixmap.fromImage(self.image).scaled(self.size(), Qt.KeepAspectRatio)
        # self.setPixmap(pixmap)
        self.setPixmap(QPixmap.fromImage(self.image))

    def resizeEvent(self, event):
        new_height = self.height()
        new_grid_height = new_height / float(self.slice_size[1])
        mul_height = new_grid_height / self.grid[1]
        # mul_width = mul_height

        new_width = self.width()
        new_grid_width = new_width / float(self.slice_size[0])
        mul_width = new_grid_width / self.grid[0]

        # self.grid = np.array(self.grid) * mul_height
        # self.grid =  np.array(self.grid)
        # self.grid[0] *= mul_width
        # self.grid[1] *= mul_height
        self.grid_res[0] = self.grid[0] * mul_width
        self.grid_res[1] = self.grid[1] * mul_height
        # print self.grid, self.grid_res

        self.resizeSlice()
        self.updateSlice()

    def set_width(self, new_width):
        new_grid = new_width / float(self.slice_size[0])
        mul = new_grid / self.grid[0]

        self.grid = np.array(self.grid) * mul
        self.resizeSlice()
        self.updateSlice()

    def getCW(self):
        return self.cw

    def setCW(self, val, key):
        self.cw[key] = val

    def setScrollFun(self, fun):
        self.scroll_fun = fun

    def wheelEvent(self, event):
        step = 4
        d = event.delta()
        nd = d / abs(d)
        self.circle_r += step * nd
        self.circle_r = max(self.circle_r, 1)
        self.circle_strel = np.nonzero(skimor.disk(self.circle_r))
        self.updateSlice()
        # if self.scroll_fun is not None:
        #     self.scroll_fun(-nd)

    def mouseMoveEvent(self, QMouseEvent):
        center = list(self.gridPosition(QMouseEvent.pos()))
        self.mouse_cursor = center

        circle_x = (self.circle_strel[1] + center[0] - self.circle_r).astype(np.int)
        circle_y = (self.circle_strel[0] + center[1] - self.circle_r).astype(np.int)

        idx_x = (circle_x >= 0) * (circle_x < self.slice_size[0])
        idx_y = (circle_y >= 0) * (circle_y < self.slice_size[1])
        idx = idx_x * idx_y
        circle_x = circle_x[np.nonzero(idx)]
        circle_y = circle_y[np.nonzero(idx)]

        self.circle_m = np.ravel_multi_index((circle_y, circle_x), self.slice_size[::-1])

        if self.area_hist_widget is not None:
            self.circle_area_data = self.ctslice[(circle_x, circle_y)]
            self.area_hist_widget.set_data(self.circle_area_data)

        self.updateSlice()

    # def mousePressEvent(self, QMouseEvent):
    #     print 'click'
    #     coords = list(self.gridPosition(QMouseEvent.pos()))
    #     print 'pos = ', coords, ', data = ', self.ctslice[coords[0], coords[1]]
        # if self.show_mode == self.SHOW_LABELS:
        #     label = self.main.actual_data
        # return coords

    def myMousePressEvent(self, QMouseEvent):
        print 'myMousePressEvent: ',
        coords = list(self.gridPosition(QMouseEvent.pos()))
        density = self.ctslice[coords[0], coords[1]]
        print 'pos = ', coords, ', data = ', density
        self.mouseClickSignal.emit(coords, density)
        # self.mousePressEvent = None
        # self.mousePressEvent = self.myEmptyMousePressEvent

    def myEmptyMousePressEvent(self, QMouseEvevnt):
        print 'dummy mouse press event'

if __name__ == '__main__':
    from lession_editor_GUI_slim2 import Ui_MainWindow

    class Window(QMainWindow):
        def __init__(self, img, parent=None):
            QWidget.__init__(self, parent)

            v_size = np.array([1, 1, 1])

            viewer = SliceBox(img.shape, v_size)
            viewer.setCW(0, 'c')
            viewer.setCW(1, 'w')
            viewer.setSlice(img)
            viewer.circle_active = True
            viewer.setMouseTracking(True)

            layout = QHBoxLayout()
            layout.addWidget(viewer)

            ui = Ui_MainWindow()
            ui.setupUi(self)
            ui.viewer_F.setLayout(layout)

    app = QApplication(sys.argv)

    img = skiio.imread('/home/tomas/Dropbox/images/puma.png', as_grey=True)
    img = img.transpose()
    win = Window(img)
    win.show()

    sys.exit(app.exec_())